@@ -1,6 +1,6 @@
 bic.glm.fsreg <- function( target, dataset, wei = NULL, tol = 0, heavy = FALSE, robust = FALSE, ncores = 1) {
 
-  p <- ncol(dataset)  ## number of variables
+  p <- dim(dataset)[2]  ## number of variables
   bico <- numeric( p )
   moda <- list()
   k <- 1   ## counter
@@ -12,43 +12,28 @@ bic.glm.fsreg <- function( target, dataset, wei = NULL, tol = 0, heavy = FALSE,
 
   
     #check for NA values in the dataset and replace them with the variable median or the mode
-  if(any(is.na(dataset)) == TRUE)
-  {
-
-  
-    #dataset = as.matrix(dataset);
+  if( any(is.na(dataset)) ) {
     warning("The dataset contains missing values (NA) and they were replaced automatically by the variable (column) median (for numeric) or by the most frequent level (mode) if the variable is factor")
-    
     if (class(dataset) == "matrix")  {
-    
-       dataset = apply(dataset, 2, function(x){ x[which(is.na(x))] = median(x, na.rm = TRUE) ; return(x)}) 
-              
-    }else{
-	
-    poia <- which( is.na(dataset), arr.ind = TRUE )[2]
- 	  for( i in poia )
-      {
-          xi = dataset[, i]
-          if(class(xi) == "numeric")
-          {                    
-            xi[ which( is.na(xi) ) ] = median(xi, na.rm = TRUE) 
-          } else if ( class(xi) == "factor" ) {
-            xi[ which( is.na(xi) ) ] = levels(xi)[ which.max( as.vector( table(xi) ) )]
-          }
-          dataset[, i] = xi
-        }
+      dataset <- apply( dataset, 2, function(x){ x[which(is.na(x))] = median(x, na.rm = TRUE) ; return(x) } ) 
+    } else {
+      poia <- which( is.na(dataset), arr.ind = TRUE )[2]
+      for( i in poia )  {
+        xi <- dataset[, i]
+        if ( is.numeric(xi) ) {                    
+          xi[ which( is.na(xi) ) ] <- median(xi, na.rm = TRUE) 
+        } else if ( is.factor( xi ) )     xi[ which( is.na(xi) ) ] <- levels(xi)[ which.max( as.vector( table(xi) ) )]
+        dataset[, i] <- xi
+      }
     }
-    
   }
   
   if ( heavy )  con <- log(n)
-  
   #########
   ## if it is binomial or poisson regression
   #########
-  
   la <- length( Rfast::sort_unique(target) ) 
-  
+
   if ( la == 2  || ( sum( round(target) - target ) == 0  &  la > 2 ) ) {
 
    if ( la == 2 ) {
@@ -62,11 +47,11 @@ bic.glm.fsreg <- function( target, dataset, wei = NULL, tol = 0, heavy = FALSE,
     durat <- proc.time()
     
     #if ( robust == FALSE ) {
-	  if ( heavy == FALSE ) {
-        ini = BIC( glm( target ~ 1, family = oiko, weights = wei ) )   ## initial BIC
+	  if ( !heavy ) {
+        ini = BIC( glm( target ~ 1, family = oiko, weights = wei, y = FALSE, model = FALSE ) )   ## initial BIC
 	  } else {
-	    inimod <- speedglm::speedglm( target ~ 1, family = oiko, weights = wei )
-	    ini <- inimod$logLik + con   ## initial BIC
+	    inimod <- speedglm::speedglm( target ~ 1, data = data.frame(dataset), family = oiko, weights = wei )
+	    ini <-  - 2 * inimod$logLik + con   ## initial BIC
 	    ci_test <- "testIndSpeedglm"
 	  }	
     #} else {
@@ -76,17 +61,17 @@ bic.glm.fsreg <- function( target, dataset, wei = NULL, tol = 0, heavy = FALSE,
     if (ncores <= 1) {
 	
       #if ( robust == FALSE ) {  ## Non robust
-	    if ( heavy == FALSE ) {
+	    if ( !heavy ) {
           for (i in 1:p) {
-            mi <- glm( target ~ dataset[, i], family = oiko, weights = wei )
+            mi <- glm( target ~ dataset[, i], family = oiko, weights = wei, y = FALSE, model = FALSE )
             bico[i] <- BIC( mi )
           }
-		} else {
-      for (i in 1:p) { 
-        mi <- speedglm::speedglm( target ~ dataset[, i], data = as.data.frame( dataset[, i] ), family = oiko, weights = wei )
-	      bico[i] <- mi$logLik + length( coef(mi) ) * con   ## initial BIC
-		  }	
-		}  
+		  } else {
+        for (i in 1:p) { 
+          mi <- speedglm::speedglm( target ~ dataset[, i], data = data.frame( dataset[, i] ), family = oiko, weights = wei )
+	        bico[i] <-  - 2 * mi$logLik + length( coef(mi) ) * con   ## initial BIC
+		    }	
+		  }  
 
       # } else {  ## Robust
       #   for (i in 1:p) {
@@ -97,13 +82,11 @@ bic.glm.fsreg <- function( target, dataset, wei = NULL, tol = 0, heavy = FALSE,
         
       mat <- cbind(1:p, bico)
 
-      if( any( is.na(mat) ) ) {
-        mat[ which( is.na(mat) ) ] = ini
-      }
+      if( any( is.na(mat) ) )   mat[ which( is.na(mat) ) ] = ini
 
     } else {
       #if ( robust == FALSE ) {  ## Non robust
-	    if ( heavy == FALSE ) {
+	    if ( !heavy ) {
           cl <- makePSOCKcluster(ncores)
           registerDoParallel(cl)
           bico <- numeric(p)
@@ -112,18 +95,17 @@ bic.glm.fsreg <- function( target, dataset, wei = NULL, tol = 0, heavy = FALSE,
             bico[i] <- BIC( ww )
           }
           stopCluster(cl)
-		} else {
-		  cl <- makePSOCKcluster(ncores)
-          registerDoParallel(cl)
-          bico <- numeric(p)
-          mod <- foreach( i = 1:p, .combine = rbind, .export = "speedglm", .packages = "speedglm") %dopar% {
-            ww <- speedglm( target ~ dataset[, i], data = as.data.frame(dataset[, i]), family = oiko, weights = wei )
- 	        bico[i] <- ww$logLik + length( coef(ww) ) * con   ## initial BIC
-
-          }
-          stopCluster(cl)
+		  }  else {
+		     cl <- makePSOCKcluster(ncores)
+         registerDoParallel(cl)
+         bico <- numeric(p)
+         mod <- foreach( i = 1:p, .combine = rbind, .export = "speedglm", .packages = "speedglm") %dopar% {
+            ww <- speedglm( target ~ dataset[, i], data = data.frame(dataset[, i]), family = oiko, weights = wei )
+ 	          bico[i] <-  -2 * ww$logLik + length( coef(ww) ) * con   ## initial BIC
+         }
+         stopCluster(cl)
 		
-		}  
+		  }  
 
       # } else {  ## Robust
       #   cl <- makePSOCKcluster(ncores)
@@ -137,10 +119,7 @@ bic.glm.fsreg <- function( target, dataset, wei = NULL, tol = 0, heavy = FALSE,
       # }
 
       mat <- cbind(1:p, mod)
-
-      if ( any( is.na(mat) ) ) {
-        mat[ which( is.na(mat) ) ] = ini
-      }
+      if ( any( is.na(mat) ) )    mat[ which( is.na(mat) ) ] = ini
       
     }
 
@@ -153,35 +132,29 @@ bic.glm.fsreg <- function( target, dataset, wei = NULL, tol = 0, heavy = FALSE,
 
       info[1, ] <- mat[sel, ]
       mat <- mat[-sel, ]
-      if ( !is.matrix(mat) ) {
-        mat <- matrix(mat, ncol = 2) 
-      }
-      mat <- mat[ order( mat[, 2] ), ]
-
+      if ( !is.matrix(mat) )    mat <- matrix(mat, ncol = 2) 
       #if ( robust == FALSE ) {
-
-	    if ( heavy == FALSE ) {
-          mi <- glm( target ~ dataset[, sel], family = oiko, weights = wei )
+	    if ( !heavy ) {
+          mi <- glm( target ~ dataset[, sel], family = oiko, weights = wei, y = FALSE, model = FALSE )
           tool[1] <- BIC( mi )
 		   } else {
-          mi <- speedglm::speedglm( target ~ dataset[, sel], data = as.data.frame(dataset[, sel]), family = oiko, weights = wei )
-          tool[1] <- mi$logLik + length( coef(mi) ) * con   ## initial BIC
+          mi <- speedglm::speedglm( target ~ dataset[, sel], data = data.frame(dataset[, sel]), family = oiko, weights = wei )
+          tool[1] <-  - 2 * mi$logLik + length( coef(mi) ) * con   ## initial BIC
 		   }  
       #} else {
       #  mi = robust::glmRob( target ~ dataset[, sel], family = oiko, maxit = maxit )
       #  tool[1] <- mi$deviance + length( coef( mi ) ) * log(n)
       #  if ( is.na(tool[1]) )  tool[1] <- ini
       #}
-
       moda[[ 1 ]] <- mi
+    }  else  {
+      info <- info  
+      sela <- NULL
     }
-
-
     ######
     ###     k equals 2
     ######
 
-
     if ( length(moda) > 0  &  nrow(mat) > 0 ) {
 
       k <- 2
@@ -191,15 +164,15 @@ bic.glm.fsreg <- function( target, dataset, wei = NULL, tol = 0, heavy = FALSE,
       if ( ncores <= 1 ) {
         bico <- numeric( pn )
         #if ( robust == FALSE ) {  ## Non robust
-		    if ( heavy == FALSE ) {
+		    if ( !heavy ) {
           for ( i in 1:pn ) {
-            ma <- glm( target ~., data = as.data.frame( dataset[, c(sel, mat[i, 1]) ] ), family = oiko, weights = wei )
+            ma <- glm( target ~., data = data.frame( dataset[, c(sel, mat[i, 1]) ] ), family = oiko, weights = wei, y = FALSE, model = FALSE )
             bico[i] <- BIC( ma )
           }
         } else {
           for ( i in 1:pn ) {
-            ma <- speedglm::speedglm( target ~., data = as.data.frame( dataset[, c(sel, mat[i, 1]) ] ), family = oiko, weights = wei )
-            bico[i] <- ma$logLik + length( coef(ma) ) * con
+            ma <- speedglm::speedglm( target ~., data = data.frame( dataset[, c(sel, mat[i, 1]) ] ), family = oiko, weights = wei )
+            bico[i] <-  - 2 * ma$logLik + length( coef(ma) ) * con
           }		  
 		    }
         # } else {  ## Robust
@@ -208,32 +181,30 @@ bic.glm.fsreg <- function( target, dataset, wei = NULL, tol = 0, heavy = FALSE,
         #     bico[i] <- ma$deviance + length( coef( ma ) ) * log(n)
         #   }
         # }
-
         mat[, 2] <- bico
 
       } else {
 
         #if ( robust == FALSE ) {  ## Non robust
-		  if ( heavy == FALSE ) {
-            cl <- makePSOCKcluster(ncores)
-            registerDoParallel(cl)
-            bico <- numeric(pn)
-            mod <- foreach( i = 1:pn, .combine = rbind) %dopar% {
-              ww <- glm( target ~ dataset[, sel ] + dataset[, mat[i, 1] ], family = oiko, weights = wei )
-              bico[i] <- BIC( ww )
-            }
+		  if ( !heavy ) {
+          cl <- makePSOCKcluster(ncores)
+          registerDoParallel(cl)
+          bico <- numeric(pn)
+          mod <- foreach( i = 1:pn, .combine = rbind) %dopar% {
+            ww <- glm( target ~ dataset[, sel ] + dataset[, mat[i, 1] ], family = oiko, weights = wei )
+            bico[i] <- BIC( ww )
+          }
             stopCluster(cl)
 		  } else {
-            cl <- makePSOCKcluster(ncores)
-            registerDoParallel(cl)
-            bico <- numeric(pn)
-            mod <- foreach( i = 1:pn, .combine = rbind, export = "speedglm", .packages = "speedglm") %dopar% {
-              ww <- speedglm( target ~ dataset[, sel ] + dataset[, mat[i, 1] ], data = as.data.frame(dataset), family = oiko, weights = wei )
-              bico[i] <- ww$logLik + length( coef(ww) ) * con
-            }
+          cl <- makePSOCKcluster(ncores)
+          registerDoParallel(cl)
+          bico <- numeric(pn)
+          mod <- foreach( i = 1:pn, .combine = rbind, export = "speedglm", .packages = "speedglm") %dopar% {
+            ww <- speedglm( target ~ dataset[, sel ] + dataset[, mat[i, 1] ], data = data.frame(dataset), family = oiko, weights = wei )
+            bico[i] <-  - 2 * ww$logLik + length( coef(ww) ) * con
+          }
             stopCluster(cl)		  
 		  }	
-
         # } else {  ## Robust
         #   cl <- makePSOCKcluster(ncores)
         #   registerDoParallel(cl)
@@ -245,9 +216,7 @@ bic.glm.fsreg <- function( target, dataset, wei = NULL, tol = 0, heavy = FALSE,
         # 
         #   stopCluster(cl)
         # }
-
         mat[, 2] <- mod
-
       }
 
       ina <- which.min( mat[, 2] )
@@ -261,24 +230,16 @@ bic.glm.fsreg <- function( target, dataset, wei = NULL, tol = 0, heavy = FALSE,
         info <- rbind(info, mat[ina, ] )
         sela <- info[, 1]
         mat <- mat[-ina , ]
-        if ( !is.matrix(mat) ) {
-          mat <- matrix(mat, ncol = 2) 
-        }
-        mat <- mat[ order( mat[, 2] ), ]
-
-      }
-
-     if ( sum( info[2, ] -  c( -10, 1e300 ) ) == 0 ) {
-       info <- matrix( info[1, ], ncol = 2)
+        if ( !is.matrix(mat) )   mat <- matrix(mat, ncol = 2) 
      }
 
+     if ( sum( info[2, ] -  c( -10, 1e300 ) ) == 0 )  info <- matrix( info[1, ], ncol = 2)
+
    }
 
-
-      #########
-      ####      k is greater than 2
-      #########
-
+   #########
+   ####      k is greater than 2
+   #########
 
     if ( nrow(info) > 1  &  nrow(mat) > 0 ) {
       while ( ( k < n - 10 ) & ( tool[ k - 1 ] - tool[ k ] > tol ) & ( nrow(mat) > 0 ) ) {
@@ -288,17 +249,17 @@ bic.glm.fsreg <- function( target, dataset, wei = NULL, tol = 0, heavy = FALSE,
 
         if (ncores <= 1) {
           #if ( robust == FALSE ) {  ## Non robust
-		  if ( heavy == FALSE ) {
+		    if ( !heavy ) {
             for ( i in 1:pn ) {
-              ma <- glm( target ~., data = as.data.frame( dataset[, c(sela, mat[i, 1]) ] ), family = oiko, weights = wei )
+              ma <- glm( target ~., data = as.data.frame( dataset[, c(sela, mat[i, 1]) ] ), family = oiko, weights = wei, y = FALSE, model = FALSE )
               mat[i, 2] <- BIC( ma )
             }
           } else {
             for ( i in 1:pn ) {
-              ma <- speedglm::speedglm( target ~., data = as.data.frame( dataset[, c(sela, mat[i, 1]) ] ), family = oiko, weights = wei )
-              mat[i, 2] <- ma$logLik + length( coef(ma) ) * con
+              ma <- speedglm::speedglm( target ~., data = data.frame( dataset[, c(sela, mat[i, 1]) ] ), family = oiko, weights = wei )
+              mat[i, 2] <-  - 2 * ma$logLik + length( coef(ma) ) * con
             }		  
-		  }   
+		      }   
           # } else {  ## Robust
           #   for ( i in 1:pn ) {
           #     ma <- robust::glmRob( target ~., data = as.data.frame( dataset[, c(sela, mat[i, 1]) ] ), family = oiko, maxit = maxit )
@@ -308,7 +269,7 @@ bic.glm.fsreg <- function( target, dataset, wei = NULL, tol = 0, heavy = FALSE,
 
         } else {
           #if ( robust == FALSE ) {  ## Non robust
-		      if ( heavy == FALSE ) {
+		      if ( !heavy ) {
             cl <- makePSOCKcluster(ncores)
             registerDoParallel(cl)
             bico <- numeric(pn)
@@ -322,8 +283,8 @@ bic.glm.fsreg <- function( target, dataset, wei = NULL, tol = 0, heavy = FALSE,
             registerDoParallel(cl)
             bico <- numeric(pn)
             mod <- foreach( i = 1:pn, .combine = rbind, .export = "speedglm", .packages = "speedglm") %dopar% {
-              ww <- speedglm( target ~., data = as.data.frame( dataset[, c(sela, mat[i, 1]) ] ), family = oiko, weights = wei )
-              bico[i] <- ww$logLik + length( coef(ww) ) * con
+              ww <- speedglm( target ~., data = data.frame( dataset[, c(sela, mat[i, 1]) ] ), family = oiko, weights = wei )
+              bico[i] <-  - 2 * ww$logLik + length( coef(ww) ) * con
             }
             stopCluster(cl)		  
 		      }
@@ -339,7 +300,6 @@ bic.glm.fsreg <- function( target, dataset, wei = NULL, tol = 0, heavy = FALSE,
           # 
           #   stopCluster(cl)
           # }
-        
           mat[, 2] <- mod
 
         }
@@ -352,15 +312,11 @@ bic.glm.fsreg <- function( target, dataset, wei = NULL, tol = 0, heavy = FALSE,
           tool[k] <- 1e+300
 
         } else {
-
           tool[k] = mat[ina, 2]
           info <- rbind(info, mat[ina, ] )
           sela <- info[, 1]
           mat <- mat[-ina , ]
-          if ( !is.matrix(mat) ) {
-            mat <- matrix(mat, ncol = 2) 
-          }
-          mat <- mat[ order( mat[, 2] ), ]
+          if ( !is.matrix(mat) )  mat <- matrix(mat, ncol = 2) 
 
         }
 
@@ -385,20 +341,17 @@ bic.glm.fsreg <- function( target, dataset, wei = NULL, tol = 0, heavy = FALSE,
   
     durat <- proc.time()
       
-	  if ( min(target) > 0  &  max(target) < 1 ) {
-	    target <- log( target / (1 - target) )
+	  if ( min(target) > 0  &  max(target) < 1 )    target <- log( target / (1 - target) )
 	  
-	  }
-	  
-      if ( robust == FALSE ) {  ## Non robust
-	      if ( heavy == FALSE ) {
+      if ( !robust ) {  ## Non robust
+	      if ( !heavy ) {
 	        ci_test <- "testIndReg"
-          mi <- lm( target ~ 1 )
+          mi <- lm( target ~ 1, y = FALSE, model = FALSE )
           ini <- BIC( mi )
 	    	} else {
 	    	  ci_test <- "testIndSpeedglm"
-		      mi <- speedglm::speedlm(target ~ 1)
-		      ini <- mi$logLik + con
+		      mi <- speedglm::speedlm(target ~ 1, data = data.frame(dataset) )
+		      ini <- BIC(mi)
 		    }  
 
       } else {  ## Robust
@@ -409,35 +362,35 @@ bic.glm.fsreg <- function( target, dataset, wei = NULL, tol = 0, heavy = FALSE,
 
       if (ncores <= 1) {
 	
-      if ( robust == FALSE ) {  ## Non robust
-	      if ( heavy == FALSE ) {
+      if ( !robust ) {  ## Non robust
+	      if ( !heavy ) {
           for (i in 1:p) {
-            mi <- lm( target ~ dataset[, i] )
+            mi <- lm( target ~ dataset[, i], y = FALSE, model = FALSE )
             bico[i] <- BIC( mi )
           }
         } else {
           for (i in 1:p) {
-            mi <- speedglm::speedlm( target ~ dataset[, i], data = as.data.frame(dataset[, i]) )
-            bico[i] <- mi$logLik + length( coef(mi) ) * con
+            mi <- speedglm::speedlm( target ~ dataset[, i], data = data.frame(dataset[, i]) )
+            bico[i] <- BIC(mi)
           }		
 		    }
       } else {  ## Robust
         for (i in 1:p) {
-         mi = MASS::rlm( target ~ dataset[, i], maxit = 2000, weights = wei )
-         bico[i] = BIC(mi)
+         mi <- MASS::rlm( target ~ dataset[, i], maxit = 2000, weights = wei )
+         bico[i] <- BIC(mi)
         }
       }
 
      mat <- cbind(1:p, bico)
 
     } else {
-      if ( robust == FALSE ) {  ## Non robust
-        if( heavy == FALSE ) {
+      if ( !robust ) {  ## Non robust
+        if( !heavy ) {
           cl <- makePSOCKcluster(ncores)
           registerDoParallel(cl)
           bico <- numeric(p)
           mod <- foreach( i = 1:p, .combine = rbind ) %dopar% {
-            ww <- lm( target ~ dataset[, i], weights = wei )
+            ww <- lm( target ~ dataset[, i], weights = wei, y = FALSE, model = FALSE )
             bico[i] <- BIC( ww )
           }
           stopCluster(cl)
@@ -446,8 +399,8 @@ bic.glm.fsreg <- function( target, dataset, wei = NULL, tol = 0, heavy = FALSE,
           registerDoParallel(cl)
           bico <- numeric(p)
           mod <- foreach( i = 1:p, .combine = rbind, .export = "speedlm", .packages = "speedglm" ) %dopar% {
-            ww <- speedglm::speedlm( target ~ dataset[, i], data = as.data.frame(dataset[, i]), weights = wei )
-            bico[i] <- ww$logLik + length( coef(ww) ) * con
+            ww <- speedglm::speedlm( target ~ dataset[, i], data = data.frame(dataset[, i]), weights = wei )
+            bico[i] <- BIC(ww)
           }
           stopCluster(cl)		
 		}
@@ -473,13 +426,13 @@ bic.glm.fsreg <- function( target, dataset, wei = NULL, tol = 0, heavy = FALSE,
 
     if ( mat[sel, 2] < ini ) {
 	
-      if ( robust == FALSE ) {
-	      if ( heavy == FALSE ) {
-          mi <- lm( target ~ dataset[, sel], weights = wei)
+      if ( !robust ) {
+	      if ( !heavy ) {
+          mi <- lm( target ~ dataset[, sel], weights = wei, y = FALSE, model = FALSE )
           tool[1] <- BIC( mi )
 		    } else {
-          mi <- speedglm::speedlm( target ~ dataset[, sel], data = as.data.frame(dataset[, sel]), weight = wei)
-          tool[1] <- mi$logLik + length( coef(mi) ) * con	
+          mi <- speedglm::speedlm( target ~ dataset[, sel], data = data.frame(dataset[, sel]), weight = wei)
+          tool[1] <- BIC(mi)	
 		    }
       } else {
         mi <- MASS::rlm( target ~ dataset[, sel], maxit = 2000, weights = wei )
@@ -488,13 +441,11 @@ bic.glm.fsreg <- function( target, dataset, wei = NULL, tol = 0, heavy = FALSE,
       moda[[ 1 ]] <- mi
       info[1, ] <- mat[sel, ]
       mat <- mat[-sel, ]
-      if ( !is.matrix(mat) ) {
-        mat <- matrix(mat, ncol = 2) 
-      }
-      mat <- mat[ order( mat[, 2] ), ]    
-	  
-	  }
-
+      if ( !is.matrix(mat) )   mat <- matrix(mat, ncol = 2) 
+    }  else  {
+      info <- info  
+      sela <- NULL
+    }
     ######
     ###     k equals 2
     ######
@@ -507,16 +458,16 @@ bic.glm.fsreg <- function( target, dataset, wei = NULL, tol = 0, heavy = FALSE,
 
       if ( ncores <= 1 ) {
 	  
-        if ( robust == FALSE ) {
-		      if ( heavy == FALSE ) {
+        if ( !robust ) {
+		      if ( !heavy ) {
              for (i in 1:pn) {
-               ma <- lm( target ~., data = as.data.frame( dataset[, c(sel, mat[i, 1]) ] ), weights = wei )
+               ma <- lm( target ~., data = data.frame( dataset[, c(sel, mat[i, 1]) ] ), weights = wei, y = FALSE, model = FALSE )
                mat[i, 2] <- BIC( ma )
              }
 		      } else {
              for (i in 1:pn) {
-               ma <- speedglm::speedlm( target ~., data = as.data.frame( dataset[, c(sel, mat[i, 1]) ] ), weights = wei )
-               mat[i, 2] <- ma$logLik + length( coef(ma) ) * con
+               ma <- speedglm::speedlm( target ~., data = data.frame( dataset[, c(sel, mat[i, 1]) ] ), weights = wei )
+               mat[i, 2] <- BIC(ma)
              }		   
 		      } 	 
         } else {
@@ -528,13 +479,13 @@ bic.glm.fsreg <- function( target, dataset, wei = NULL, tol = 0, heavy = FALSE,
 
       } else {
 
-       if ( robust == FALSE ) {  ## Non robust
-	       if ( heavy == FALSE ) {
+       if ( !robust ) {  ## Non robust
+	       if ( !heavy ) {
            cl <- makePSOCKcluster(ncores)
            registerDoParallel(cl)
            bico <- numeric(pn)
            mod <- foreach( i = 1:pn, .combine = rbind) %dopar% {
-             ww <- lm( target ~., data = as.data.frame( dataset[, c(sel, mat[i, 1]) ] ), weights = wei )
+             ww <- lm( target ~., data = as.data.frame( dataset[, c(sel, mat[i, 1]) ] ), weights = wei, y = FALSE, model = FALSE )
              bico[i] <- BIC( ww )
            }
            stopCluster(cl)
@@ -543,8 +494,8 @@ bic.glm.fsreg <- function( target, dataset, wei = NULL, tol = 0, heavy = FALSE,
            registerDoParallel(cl)
            bico <- numeric(pn)
            mod <- foreach( i = 1:pn, .combine = rbind, .export = "speedlm", .packages = "speedglm") %dopar% {
-             ww <- speedglm::speedlm( target ~., data = as.data.frame( dataset[, c(sel, mat[i, 1]) ] ), weights = wei )
-             bico[i] <- ww$logLik + length( coef(ww) ) * con
+             ww <- speedglm::speedlm( target ~., data = data.frame( dataset[, c(sel, mat[i, 1]) ] ), weights = wei )
+             bico[i] <- BIC(ww)
            }
            stopCluster(cl)		 
 		     }  
@@ -575,14 +526,10 @@ bic.glm.fsreg <- function( target, dataset, wei = NULL, tol = 0, heavy = FALSE,
         info <- rbind(info, mat[ina, ] )
         sela <- info[, 1]
         mat <- mat[-ina , ]
-        if ( !is.matrix(mat) ) mat <- matrix(mat, ncol = 2) 
-        mat <- mat[ order( mat[, 2] ), ]
-
+        if ( !is.matrix(mat) )  mat <- matrix(mat, ncol = 2) 
       }
 
-      if ( sum( info[2, ] -  c( -10, 1e300 ) ) == 0 ) {
-        info <- matrix( info[1, ], ncol = 2)
-      }
+      if ( sum( info[2, ] -  c( -10, 1e300 ) ) == 0 )  info <- matrix( info[1, ], ncol = 2)
 
     }
 
@@ -598,16 +545,16 @@ bic.glm.fsreg <- function( target, dataset, wei = NULL, tol = 0, heavy = FALSE,
 
         if ( ncores <= 1 ) {
 		
-           if ( robust == FALSE ) {
-		         if ( heavy == FALSE ) { 
+           if ( !robust ) {
+		         if ( !heavy ) { 
                for ( i in 1:pn ) {
-                 ma <- lm( target ~., data = as.data.frame( dataset[, c(sela, mat[i, 1]) ] ), weights = wei )
+                 ma <- lm( target ~., data = data.frame( dataset[, c(sela, mat[i, 1]) ] ), weights = wei, y = FALSE, model = FALSE )
                  mat[i, 2] <- BIC( ma )
                }
 			       } else {
                for ( i in 1:pn ) {
-                 ma <- speedglm::speedlm( target ~., data = as.data.frame( dataset[, c(sela, mat[i, 1]) ] ), weights = wei )
-                 mat[i, 2] <- ww$logLik + length( coef(ww) ) * con
+                 ma <- speedglm::speedlm( target ~., data = data.frame( dataset[, c(sela, mat[i, 1]) ] ), weights = wei )
+                 mat[i, 2] <- BIC(ww)
                }			 
 			       }  
 
@@ -619,13 +566,13 @@ bic.glm.fsreg <- function( target, dataset, wei = NULL, tol = 0, heavy = FALSE,
            }
 
         } else {
-           if ( robust == FALSE ) {  ## Non robust
-		         if ( heavy == FALSE ) {
+           if ( !robust ) {  ## Non robust
+		         if ( !heavy ) {
                cl <- makePSOCKcluster(ncores)
                registerDoParallel(cl)
                bico <- numeric(pn)
                mod <- foreach( i = 1:pn, .combine = rbind) %dopar% {
-                 ww <- lm( target ~., data = as.data.frame( dataset[, c(sela, mat[i, 1]) ] ), weights = wei )
+                 ww <- lm( target ~., data = data.frame( dataset[, c(sela, mat[i, 1]) ] ), weights = wei, y = FALSE, model = FALSE )
                  bico[i] <- BIC( ww )
                }
                stopCluster(cl)
@@ -634,8 +581,8 @@ bic.glm.fsreg <- function( target, dataset, wei = NULL, tol = 0, heavy = FALSE,
                registerDoParallel(cl)
                bico <- numeric(pn)
                mod <- foreach( i = 1:pn, .combine = rbind, .export = "speedlm", .packages = "speedlm") %dopar% {
-                 ww <- speedglm::speedlm( target ~., data = as.data.frame( dataset[, c(sela, mat[i, 1]) ] ), weights = wei )
-                 bico[i] <- ww$logLik + length( coef(ww) ) * con
+                 ww <- speedglm::speedlm( target ~., data = data.frame( dataset[, c(sela, mat[i, 1]) ] ), weights = wei )
+                 bico[i] <- BIC(ww)
                }
                stopCluster(cl)			
 			       }  
@@ -646,7 +593,7 @@ bic.glm.fsreg <- function( target, dataset, wei = NULL, tol = 0, heavy = FALSE,
              bico <- numeric(pn)
              mod <- foreach( i = 1:p, .combine = rbind, export = "rlm", .packages = "MASS" ) %dopar% {
                ww <- rlm( target ~., data = data.frame( dataset[, c(sela, mat[i, 1]) ] ), maxit = 2000, weights = wei )
-               bico[i] = BIC( ww )
+               bico[i] <- BIC( ww )
              }
              stopCluster(cl)
            }
@@ -668,10 +615,7 @@ bic.glm.fsreg <- function( target, dataset, wei = NULL, tol = 0, heavy = FALSE,
            info <- rbind(info, mat[ina, ] )
            sela <- info[, 1]
            mat <- mat[-ina , ]
-           if ( !is.matrix(mat) ) {
-             mat <- matrix(mat, ncol = 2) 
-           }
-           mat <- mat[ order( mat[, 2] ), ]
+           if ( !is.matrix(mat) )   mat <- matrix(mat, ncol = 2) 
 
         }
 
@@ -692,29 +636,23 @@ bic.glm.fsreg <- function( target, dataset, wei = NULL, tol = 0, heavy = FALSE,
       colnames(xx) <- paste("V", sela, sep = "")
       if ( d == 1 ) {
 
-       models <- NULL
+        models <- NULL
         xx <- as.data.frame( dataset[, sela] )
         colnames(xx) <- paste("V", sela, sep = "")
 
         if ( is.null(oiko) ) {
 		
-           if ( robust == FALSE ) {
-		     if ( heavy == FALSE ) {
-               models <- final <- lm( target ~., data = as.data.frame( xx ), weights = wei )
-			 } else {
-               models <- final <- speedglm::speedlm( target ~., data = as.data.frame( xx ), weights = wei )	 
-			 }  
-           } else {
-             models <- final <- MASS::rlm( target ~., data = data.frame( xx ), maxit = 2000, weights = wei )
-           }
+           if ( !robust ) {
+		          if ( !heavy ) {
+                models <- final <- lm( target ~., data = data.frame( xx ), weights = wei, y = FALSE, model = FALSE )
+			        } else  models <- final <- speedglm::speedlm( target ~., data = as.data.frame( xx ), weights = wei )	 
+           } else  models <- final <- MASS::rlm( target ~., data = data.frame( xx ), maxit = 2000, weights = wei )
 
         } else {
           #if ( robust == FALSE ) {
-		  if ( heavy == FALSE ) {
-            models <- final <- glm( target ~., data = as.data.frame( xx ), family = oiko, weights = wei )
-		  } else {
-            models <- final <- speedglm::speedglm( target ~., data = as.data.frame( xx ), family = oiko, weights = wei )		  
-		  }	
+		      if ( !heavy ) {
+            models <- final <- glm( target ~., data = data.frame( xx ), family = oiko, weights = wei, y = FALSE, model = FALSE )
+		      } else  models <- final <- speedglm::speedglm( target ~., data = data.frame( xx ), family = oiko, weights = wei )		  
           #} else {
           #  models <- final <- robust::glmRob( target ~., data = as.data.frame( xx ), family = oiko, maxit = maxit )
           #}
@@ -723,26 +661,19 @@ bic.glm.fsreg <- function( target, dataset, wei = NULL, tol = 0, heavy = FALSE,
       } else {
         if ( is.null(oiko) ) {
 		
-          if ( robust == FALSE ) {
-		         if ( heavy == FALSE ) {
-               for (i in 1:d)  models[[ i ]] <- lm( target ~., data = as.data.frame( xx[, 1:i] ), weights = wei )
-             } else {
-               for (i in 1:d)   models[[ i ]] <- speedglm::speedlm( target ~., data = as.data.frame( xx[, 1:i] ), weights = wei )
-			       }  			   
-          } else {
-		        for (i in 1:d)  models[[ i ]] <- MASS::rlm( target ~., data = data.frame( xx[, 1:i] ), maxit = 2000, weights = wei )
-            
-          }
+          if ( !robust ) {
+		         if ( !heavy ) {
+               for (i in 1:d)  models[[ i ]] <- lm( target ~., data = data.frame( xx[, 1:i] ), weights = wei, y = FALSE, model = FALSE )
+             } else   for (i in 1:d)    models[[ i ]] <- speedglm::speedlm( target ~., data = as.data.frame( xx[, 1:i] ), weights = wei )
+          } else   for (i in 1:d)   models[[ i ]] <- MASS::rlm( target ~., data = data.frame( xx[, 1:i] ), maxit = 2000, weights = wei )
 		  
         } else {
-          if ( heavy == FALSE ) {
+          if ( !heavy ) {
             for (i in 1:d) {
             #if ( robust == FALSE ) {
-              models[[ i ]] <- glm( target ~., data = as.data.frame( xx[, 1:i] ), family = oiko, weights = wei )
+              models[[ i ]] <- glm( target ~., data = data.frame( xx[, 1:i] ), family = oiko, weights = wei, y = FALSE, model = FALSE )
 		       	}  
-		    } else {
-		       for (i in 1:d)  models[[ i ]] <- speedglm::speedglm( target ~., data = as.data.frame( xx[, 1:i] ), family = oiko, weights = wei )		
-		  }  
+		    } else   for (i in 1:d)   models[[ i ]] <- speedglm::speedglm( target ~., data = data.frame( xx[, 1:i] ), family = oiko, weights = wei )		
             #} else {
             #  models[[ i ]] <- robust::glmRob( target ~., data = as.data.frame( xx[, 1:i] ), family = oiko, maxit = maxit )
             #}
