@@ -26,82 +26,44 @@ fs.reg <- function(target, dataset, threshold = 0.05, wei = NULL, test = NULL, u
   sela <- NULL
   
   #check for NA values in the dataset and replace them with the variable median or the mode
-  if(any(is.na(dataset)) == TRUE)
-  {
-
-  
-    #dataset = as.matrix(dataset);
+  if( any(is.na(dataset)) ) {
     warning("The dataset contains missing values (NA) and they were replaced automatically by the variable (column) median (for numeric) or by the most frequent level (mode) if the variable is factor")
-    
     if (class(dataset) == "matrix")  {
-    
-       dataset = apply( dataset, 2, function(x){ x[which(is.na(x))] = median(x, na.rm = TRUE) ; return(x) } ) 
-              
-    }else{
-	
-    poia <- which( is.na(dataset), arr.ind = TRUE )[2]
- 	for( i in poia )
-      {
-          xi = dataset[, i]
-          if(class(xi) == "numeric")
-          {                    
-            xi[ which( is.na(xi) ) ] = median(xi, na.rm = TRUE) 
-          } else if ( class(xi) == "factor" ) {
-            xi[ which( is.na(xi) ) ] = levels(xi)[ which.max( as.vector( table(xi) ) )]
-          }
-          dataset[, i] = xi
-        }
+      dataset <- apply( dataset, 2, function(x){ x[which(is.na(x))] = median(x, na.rm = TRUE) ; return(x) } ) 
+    } else {
+      poia <- which( is.na(dataset), arr.ind = TRUE )[2]
+      for( i in poia )  {
+        xi <- dataset[, i]
+        if ( is.numeric(xi) ) {                    
+          xi[ which( is.na(xi) ) ] <- median(xi, na.rm = TRUE) 
+        } else if ( is.factor( xi ) )     xi[ which( is.na(xi) ) ] <- levels(xi)[ which.max( as.vector( table(xi) ) )]
+        dataset[, i] <- xi
+      }
     }
-    
   }
   
   ##################################
   # target checking and initialize #
   ##################################
   
+  if ( is.null( colnames(dataset) ) )    colnames(dataset) <- paste("X", 1:p, sep = "")
   
-  if ( is.null( colnames(dataset) ) )  {  ## checks for column names
-    colnames(dataset) <- paste("X", 1:p, sep = "")
-  }	
-  
-  ## dependent (target) variable checking if no test was given, 
-  ## but other arguments are given. For some cases, these are default cases
-  
-  if ( is.null(test) & is.null(user_test) ) {
-    
-    ## percentages
-    if ( min( target ) > 0 & max( target ) < 1 )  {  ## are they percentages?
-      target <- log( target / (1 - target) ) 
-      ci_test <- test <- "testIndReg"
-    }
+  if ( is.null(test)  &  is.null(user_test) ) {
     
     ## surival data
     if ( sum( class(target) == "Surv" ) == 1 ) {
       ci_test <- test <- "censIndCR"
-    }
-    
-    ## ordinal, multinomial or perhaps binary data
-    if ( is.factor(target) ||  is.ordered(target) || length( unique(target) ) == 2 ) {
+      
+      ## ordinal, multinomial or perhaps binary data
+    } else if ( is.factor(target) ||  is.ordered(target) || length( Rfast::sort_unique(target) ) == 2 ) {
       ci_test <- test <- "testIndLogistic"
+      
+      ## count data
+    } else if ( length( Rfast::sort_unique(target) ) > 2  &  !is.factor(target) ) {
+      if ( sum( round(target) - target ) == 0 ) {
+         ci_test <- test <- "testIndPois"
+      } else  ci_test <- test <- "testIndReg"  
     }
-    
-    ## count data
-    if ( is.vector(target) ) {
-      
-      if ( length( unique(target) ) > 2  &  sum( round(target) - target ) == 0 ) {
-        ci_test <- test <- "testIndPois"
-    
-        ## binomial regression 
-      
-      } else if ( length( unique(target) ) == 2 ) {
-        ci_test <- test <- "testIndLogistic"
-
-        ## linear regression 
-      } else if ( sum( class(target) == "numeric" ) > 0 ) {
-        ci_test <- test <- "testIndReg"  
-      }
-    }  
-    
   }
   
   #available conditional independence tests
@@ -110,7 +72,7 @@ fs.reg <- function(target, dataset, threshold = 0.05, wei = NULL, test = NULL, u
   ci_test <- test
   #cat(test)
   
-  if ( ( test == "testIndLogistic" & ( length( unique(target) ) == 2 ) )  ||  test == "testIndBinom"  || test == "testIndPois"  ||  ( sum( round(target) - target ) == 0  &  length(target) > 2 )  ) {
+  if ( ( test == "testIndLogistic" &  length( Rfast::sort_unique(target) ) == 2 )  ||  test == "testIndBinom"  || test == "testIndPois" ) {
     
     result <- glm.fsreg( target, dataset, wei = wei, threshold = exp(threshold), tol = tol, robust = robust, ncores = ncores) 
     
@@ -118,10 +80,17 @@ fs.reg <- function(target, dataset, threshold = 0.05, wei = NULL, test = NULL, u
     
     result <- lm.fsreg( target, dataset, wei = wei, threshold = exp(threshold), stopping = stopping, tol = tol, robust = robust, ncores = ncores ) 
   
-
+  } else if ( test == "testIndBeta" ) {
+    
+    result <- beta.fsreg(target, dataset, threshold = exp(threshold), wei = wei, tol = tol, ncores = ncores) 
+    
+  } else if ( test == "testIndZIP" ) {
+    
+    result <- zip.fsreg(target, dataset, threshold = exp(threshold), wei = wei, tol = tol, ncores = ncores) 
+  
   } else if ( test == "testIndSpeedglm" ) {
     
-    if ( length( unique(target) ) == 2  ||  ( sum( round(target) - target ) == 0  &  length(target) > 2 ) ) { 
+    if ( length( Rfast::sort_unique(target) ) == 2  ||  sum( round(target) - target ) == 0  ) { 
        result <- glm.fsreg( target, dataset, wei = wei, threshold = exp(threshold), tol = tol, heavy = TRUE, robust = robust, ncores = ncores) 
     
     } else  result <- lm.fsreg( target, dataset, wei = wei, threshold = exp(threshold), stopping = stopping, tol = tol, heavy = TRUE, robust = robust, ncores = ncores ) 
@@ -129,96 +98,87 @@ fs.reg <- function(target, dataset, threshold = 0.05, wei = NULL, test = NULL, u
     
   } else {
     
-    test = match.arg(test, av_models, TRUE);
+    test <- match.arg(test, av_models, TRUE);
     #convert to closure type
-    
-    if ( test == "testIndBeta" ) {
-      ci_test <- test 
-      test = betareg::betareg 
-      robust = FALSE
-      stopping = "BIC"
       
-    } else if ( test == "censIndCR" ) {
+    if ( test == "censIndCR" ) {
       ci_test <- test
-      test = survival::coxph 
-      robust = FALSE
-      stopping = "BIC"
+      test <- survival::coxph 
+      robust <- FALSE
+      stopping <- "BIC"
+      
+    } else if ( test == "censIndWR" ) {
+      ci_test <- test
+      test <- survival::survreg
+      robust <- FALSE
+      stopping <- "BIC"
       
     } else if ( test == "testIndLogistic" ) {
 	  
       ci_test <- test
       
 	    if ( is.ordered(target) )  {
-        test = ordinal::clm
-        robust = FALSE
-        stopping = "BIC"
+        test <- ordinal::clm
+        robust <- FALSE
+        stopping <- "BIC"
       } else {
-        test = nnet::multinom
-        robust = FALSE
-        stopping = "BIC"
+        test <- nnet::multinom
+        robust <- FALSE
+        stopping <- "BIC"
       }
 	  
     } else if ( test == "testIndNB" ) {
       ci_test <- test
-      test = MASS::glm.nb
-      robust = FALSE
-      stopping = "BIC"
+      test <- MASS::glm.nb
+      robust <- FALSE
+      stopping <- "BIC"
 
     } else if ( test == "testIndRQ" ) {
       ci_test <- test
-      test = quantreg::rq
-      robust = FALSE
-      stopping = "BIC"
-      
-    } else if ( test == "testIndZIP" ) {
-      ci_test <- test
-      test = pscl::zeroinfl 
-      robust = FALSE
-      stopping = "BIC"
+      test <- quantreg::rq
+      robust <- FALSE
+      stopping <- "BIC"
     
     } 
-    
-	
+
     if ( !is.null(user_test) )  {
-	  test = user_test 
-	  ci_test <- "user_test"
-	}  
-    	  
+  	  test <- user_test 
+	    ci_test <- "user_test"
+	  }  
     
     runtime <- proc.time()
     
-    devi = dof = numeric(p)
-    ini = test( target ~ 1, weights = wei ) 
-    ini =  2 * as.numeric( logLik(ini) )  ## initial 
+    devi <- dof <- numeric(p)
+    ini <- test( target ~ 1, weights = wei ) 
+    ini <-  2 * as.numeric( logLik(ini) )  ## initial 
     
     if (ncores <= 1) {
       for (i in 1:p) {
         mi <- test( target ~ dataset[, i], weights = wei )
         devi[i] <-  2 * as.numeric( logLik(mi) )
-        dof[i] = length( coef( mi ) ) 
+        dof[i] <- length( coef( mi ) ) 
       }
       
-      stat = abs( devi - ini )
-      pval = pchisq( stat, dof - 1, lower.tail = FALSE, log.p = TRUE )
+      stat <- abs( devi - ini )
+      pval <- pchisq( stat, dof - 1, lower.tail = FALSE, log.p = TRUE )
       
     } else {
       cl <- makePSOCKcluster(ncores)
       registerDoParallel(cl)
       mata <- matrix(0, p, 2)
-      mod <- foreach( i = 1:p, .combine = rbind, .packages = c("MASS", "betareg", "quantreg", "nnet", "survival", "ordinal", "pscl") ) %dopar% {
+      mod <- foreach( i = 1:p, .combine = rbind, .packages = c("MASS", "quantreg", "nnet", "survival", "ordinal") ) %dopar% {
         ww <- test( target ~ dataset[, i], weights = wei )
         mata[i, ] <- c( 2 * as.numeric( logLik(ww) ), length( coef( ww ) )  )
       }
       
       stopCluster(cl)
       
-      stat =  abs( mod[, 1] - ini )
-      pval = pchisq( stat, mod[, 2] - 1, lower.tail = FALSE, log.p = TRUE )
+      stat <-  abs( mod[, 1] - ini )
+      pval <- pchisq( stat, mod[, 2] - 1, lower.tail = FALSE, log.p = TRUE )
     }
     
     mat <- cbind(1:p, pval, stat) 
-    
-    colnames(mat)[1] <- "variables"
+    colnames(mat) <- c( "variables", "log.p-value", "stat" )
     rownames(mat) <- 1:p
     
     sel <- which.min(pval)
@@ -228,13 +188,14 @@ fs.reg <- function(target, dataset, threshold = 0.05, wei = NULL, test = NULL, u
       info[1, ] <- mat[sel, ]
       mat <- mat[-sel, ] 
       if ( !is.matrix(mat) )  mat <- matrix(mat, ncol = 3) 
-      mat <- mat[ order( mat[, 2] ), ]
       sela <- sel
       mi <- test( target ~ dataset[, sel], weights = wei )
       la <- logLik(mi)
       tool[1] <-  - 2 * as.numeric( la ) +  attr(la, "df") * con
-    
       moda[[ 1 ]] <- mi
+    }  else  {
+       info <- info  
+       sela <- NULL
     }
     
     ############
@@ -246,38 +207,34 @@ fs.reg <- function(target, dataset, threshold = 0.05, wei = NULL, test = NULL, u
       k <- 2
       pn <- p - k + 1   
 
-      ini =  2 * as.numeric( logLik( moda[[ 1 ]] ) ) 
-      do = length( coef( moda[[ 1 ]]  ) ) 
+      ini <-  2 * as.numeric( la ) 
+      do <- length( coef( moda[[ 1 ]]  ) ) 
       
       if ( ncores <= 1 ) {
-        devi = dof = numeric(pn)
+        devi <- dof <- numeric(pn)
         for ( i in 1:pn ) {
           ww <- test( target ~., data = as.data.frame( dataset[, c(sela, mat[i, 1]) ] ), weights = wei )
           devi[i] <-  2 * as.numeric( logLik(ww) )
-          dof[i] = length( coef( ww ) )          
+          dof[i] <- length( coef( ww ) )          
         }
         
-        stat = abs( devi - ini )
-        pval = pchisq( stat, dof - do, lower.tail = FALSE, log.p = TRUE )
+        stat <- abs( devi - ini )
+        pval <- pchisq( stat, dof - do, lower.tail = FALSE, log.p = TRUE )
         
       } else {
         
         cl <- makePSOCKcluster(ncores)
         registerDoParallel(cl)
-        mata = matrix(0, pn, 2)  
-        mod <- foreach( i = 1:pn, .combine = rbind, .packages = c("MASS", "betareg", "quantreg", "nnet", "survival", "ordinal", "pscl") ) %dopar% {
+        mata <- matrix(0, pn, 2)  
+        mod <- foreach( i = 1:pn, .combine = rbind, .packages = c("MASS", "quantreg", "nnet", "survival", "ordinal") ) %dopar% {
           ww <- test( target ~., data = as.data.frame( dataset[, c(sela, mat[i, 1]) ] ), weights = wei )
           mata[i, ] <- c( 2 * as.numeric( logLik(ww) ), length( coef( ww ) )  )
         }
-        
         stopCluster(cl)
         
-        stat = abs( mod[, 1] - ini )
-        pval = pchisq( stat, mod[, 2] - do, lower.tail = FALSE, log.p = TRUE )
-        
+        stat <- abs( mod[, 1] - ini )
+        pval <- pchisq( stat, mod[, 2] - do, lower.tail = FALSE, log.p = TRUE )
       }
-
-    }
     
     mat[, 2:3] <- cbind(pval, stat)
     
@@ -287,7 +244,7 @@ fs.reg <- function(target, dataset, threshold = 0.05, wei = NULL, test = NULL, u
     if ( mat[ina, 2] < threshold ) {
       ma <- test( target ~ dataset[, sela] + dataset[, sel], weights = wei )
       la <- logLik(ma)
-      tool[2]  - 2 * as.numeric( la ) +  attr(la, "df") * con
+      tool[2] <-  - 2 * as.numeric( la ) +  attr(la, "df") * con
       
       if ( tool[ 1 ] - tool[ 2 ] <= tol ) {
         info <- info
@@ -297,12 +254,13 @@ fs.reg <- function(target, dataset, threshold = 0.05, wei = NULL, test = NULL, u
         sela <- info[, 1]
         mat <- mat[-ina , ] 
         if ( !is.matrix(mat) )    mat <- matrix(mat, ncol = 3) 
-        mat <- mat[ order( mat[, 2] ), ]
         moda[[ 2 ]] <- ma
       }
       
-    } else  info <- info
-  
+    } else  info <- info  
+    
+  }
+    
   ############
   ###       k greater than 2
   ############ 
@@ -332,11 +290,10 @@ fs.reg <- function(target, dataset, threshold = 0.05, wei = NULL, test = NULL, u
           cl <- makePSOCKcluster(ncores)
           registerDoParallel(cl)
           mata = matrix(0, pn, 2)  
-          mod <- foreach( i = 1:pn, .combine = rbind, .packages = c("MASS", "betareg", "quantreg", "nnet", "survival", "ordinal", "pscl") ) %dopar% {
+          mod <- foreach( i = 1:pn, .combine = rbind, .packages = c("MASS", "quantreg", "nnet", "survival", "ordinal") ) %dopar% {
             ww <- test( target ~., data = as.data.frame( dataset[, c(sela, mat[i, 1]) ] ), weights = wei )
             mata[i, ] <- c( 2 * as.numeric( logLik(ww) ), length( coef( ww ) )  )
           }
-          
           stopCluster(cl)
           
           stat = abs( mod[, 1] - ini )
@@ -345,15 +302,14 @@ fs.reg <- function(target, dataset, threshold = 0.05, wei = NULL, test = NULL, u
         }
         
         mat[, 2:3] <- cbind(pval, stat)
-        
+      
         ina <- which.min(mat[, 2])
         sel <- mat[ina, 1]    
         
         if ( mat[ina, 2] < threshold ) {
             ma <- test( target ~., data = as.data.frame( dataset[, c(sela, sel) ] ), weights = wei )
             la <- logLik(ma)
-            tool[k]  - 2 * as.numeric( la ) +  attr(la, "df") * con
-    
+            tool[k] <-  - 2 * as.numeric( la ) +  attr(la, "df") * con
  
           if ( tool[ k - 1 ] - tool[ k  ] < tol ) {
             info <- rbind(info, c( 1e300, 0, 0 ) )
@@ -362,17 +318,11 @@ fs.reg <- function(target, dataset, threshold = 0.05, wei = NULL, test = NULL, u
             info <- rbind( info, mat[ina, ] )
             sela <- info[, 1]
             mat <- mat[-ina , ]
-            if ( !is.matrix(mat) ) {
-              mat <- matrix(mat, ncol = 3) 
-            }
-            mat <- mat[ order( mat[, 2] ), ]
-            
+            if ( !is.matrix(mat) )  mat <- matrix(mat, ncol = 3) 
             moda[[ k ]] <- ma
           } 
           
-        } else {
-          info <- rbind(info, c( 1e300, 0, 0 ) )
-        }
+        } else   info <- rbind(info, c( 1e300, 0, 0 ) )
       
     } 
     
@@ -390,26 +340,18 @@ fs.reg <- function(target, dataset, threshold = 0.05, wei = NULL, test = NULL, u
       colnames(xx) <- paste("V", sela, sep = "") 
       
       if ( d == 1 ) {
-        
         models <- NULL
         xx <- as.data.frame( dataset[, sela] )
         colnames(xx) <- paste("V", sela, sep = "") 
-        
         models[[ 1 ]] <- test( target ~., data = data.frame( xx ), weights = wei )
-
         
-      } else {
-        for (i in 1:d) {
-          models[[ i ]] <- test( target ~., data = data.frame( xx[, 1:i] ), weights = wei )
-        }
-      }
+      } else    for (i in 1:d)  models[[ i ]] <- test( target ~., data = data.frame( xx[, 1:i] ), weights = wei )
       
       final <- summary( models[[ d ]] )
-      
       info <- info[1:d, ]
       if ( d == 1 )  info <- matrix(info, nrow = 1)
       info <- cbind( info, tool[ 1:d ] ) 
-      colnames(info) <- c( "variables", "p-value", "stat", "BIC" )
+      colnames(info) <- c( "variables", "log.p-values", "stat", "BIC" )
       rownames(info) <- info[, 1]
     }
     
