@@ -1,15 +1,12 @@
-lm.fsreg <- function(target, dataset,ini = NULL, threshold = 0.05, wei = NULL, stopping = "BIC", tol = 2, heavy = FALSE, robust = FALSE, ncores = 1 ) {
-  
-  
-  ###### If there is an initia set of variables do this function
+lm.fsreg <- function(target, dataset, ini = NULL, threshold = 0.05, wei = NULL, stopping = "BIC", tol = 2, heavy = FALSE, robust = FALSE, ncores = 1 ) {  
   
+  ###### If there is an initial set of variables do this function
   if ( !is.null(ini) ) {
       result <- lm.fsreg_2(target, dataset, iniset = ini, threshold = threshold, wei = NULL, stopping = stopping, tol = tol, heavy = heavy, robust = robust, ncores = ncores) 
       
   } else {  ## else do the classical forward regression
-    
   
-  threshold = log(threshold)
+  threshold <- log(threshold)
   
   p <- dim(dataset)[2]  ## number of variables
   pval <- stat <- dof <- numeric( p )  
@@ -19,9 +16,15 @@ lm.fsreg <- function(target, dataset,ini = NULL, threshold = 0.05, wei = NULL, s
   con <- log(n)
   tool <- numeric( min(n, p) )
   
+  ## percentages
+  if ( min( target ) > 0  &  max( target ) < 1 )  target <- log( target / (1 - target) ) 
+  
   ci_test <- "testIndReg"
   
-  if ( heavy == TRUE )   robust = FALSE    ;   ci_test <- "testIndSpeedglm"
+  if ( heavy )   { 
+    robust = FALSE    
+    ci_test <- "testIndSpeedglm"
+  }
    
   # cont = robust::lmRob.control(mxr = 2000, mxf = 2000, mxs = 2000 )
 
@@ -29,22 +32,22 @@ lm.fsreg <- function(target, dataset,ini = NULL, threshold = 0.05, wei = NULL, s
     
     if (ncores <= 1) {
 
-     if ( robust == FALSE ) {  ## Non robust
+     if ( !robust ) {  ## Non robust
        
        if ( is.matrix(dataset)  &  is.null(wei) ) {
          mat <- Rfast::univglms( target, dataset, oiko = "normal", logged = TRUE ) 
-         mat <- cbind(1:p, mat[, 2], mat[, 1])
-         
+         mat <- cbind(1:p, mat)
+                  
        } else if ( is.data.frame(dataset)  &  is.null(wei) ) {
          mod <- Rfast::regression(dataset, target)
-         pval <- pf(mod[1, ], mod[2, ], n - mod[2, ] - 1, lower.tail = FALSE, log.p = FALSE)
+         pval <- pf(mod[1, ], mod[2, ], n - mod[2, ] - 1, lower.tail = FALSE, log.p = TRUE)
          mat <- cbind(1:p, mod[1, ], pval)
        
        } else {
 
-         if ( heavy == FALSE ) {
+         if ( !heavy ) {
            for (i in 1:p) {
-             ww = lm( target ~ dataset[, i], weights = wei )
+             ww = lm( target ~ dataset[, i], weights = wei, y = FALSE, model = FALSE )
              tab = anova(ww)
              stat[i] = tab[1, 4] 
              df1 = tab[1, 1]    ;   df2 = tab[2, 1]
@@ -53,29 +56,27 @@ lm.fsreg <- function(target, dataset,ini = NULL, threshold = 0.05, wei = NULL, s
 		   
          } else {
 		   for (i in 1:p) {
-             ww = speedglm:speedlm( target ~ dataset[, i], data = as.data.frame(dataset), weights = wei )
+             ww = speedglm::speedlm( target ~ dataset[, i], data = as.data.frame(dataset), weights = wei )
              suma = summary(ww)[[ 13 ]]
-             stat = suma[1]
+             stat[i] = suma[1]
              dof = suma[3]
-             pvalue = pf(stat, 1, dof, lower.tail = FALSE, log.p = TRUE)
+             pval[i] = pf(stat[i], 1, dof, lower.tail = FALSE, log.p = TRUE)
            }
 		   
 		 } 
-           mat <- cbind(1:p, pval, stat)
-		 
+        mat <- cbind(1:p, pval, stat)
        }
       
-      
      } else {  ## Robust
        for (i in 1:p) {
         # ww = robust::lmRob( target ~ dataset[, i], control = cont )
         # tab = aovlmrob( ww )
-        ww = MASS::rlm(target ~ dataset[, i ], maxit = 2000) 
+        ww = MASS::rlm( target ~ dataset[, i ], maxit = 2000, weights = wei ) 
         stat[i] = 2 * as.numeric( logLik(ww) )
         dof[i] = length( coef(ww) )
        }
 
-       fit0 = MASS::rlm(target ~ 1, maxit = 2000, weights = wei) 
+       fit0 = MASS::rlm( target ~ 1, maxit = 2000, weights = wei ) 
        stat0 = 2 * as.numeric( logLik(fit0) )
        difa = abs( stat - stat0 )
        pval = pchisq(difa, dof - 1, lower.tail = FALSE, log.p = TRUE)
@@ -84,13 +85,13 @@ lm.fsreg <- function(target, dataset,ini = NULL, threshold = 0.05, wei = NULL, s
 
 
     } else {
-      if ( robust == FALSE ) {  ## Non robust
-	     if ( heavy == FALSE ) { 
+      if ( !robust ) {  ## Non robust
+	     if ( !heavy ) { 
           cl <- makePSOCKcluster(ncores)
           registerDoParallel(cl)
           mat <- matrix(0, p, 2)
           mod <- foreach( i = 1:p, .combine = rbind ) %dopar% {
-            ww <- lm( target ~ dataset[, i], weights = wei )
+            ww <- lm( target ~ dataset[, i], weights = wei, y = FALSE, model = FALSE )
             tab <- anova( ww )
             stat[i] <- tab[1, 4] 
             df1 <- tab[1, 1]   ;  df2 = tab[2, 1]
@@ -100,22 +101,22 @@ lm.fsreg <- function(target, dataset,ini = NULL, threshold = 0.05, wei = NULL, s
           stopCluster(cl)
 		 
          } else {
-		  cl <- makePSOCKcluster(ncores)
-          registerDoParallel(cl)
-          mat <- matrix(0, p, 2)
-          mod <- foreach( i = 1:p, .combine = rbind, .export = "speedlm", .packages = "speedglm" ) %dopar% {
-            ww = speedlm( target ~ dataset[, i], data = as.data.frame(dataset), weights = wei ) 
-			      suma = summary(ww)[[ 13 ]]
-            stat[i] = suma[1]
-            dof = suma[3]
-            pval[i] = pf(stat, 1, dof, lower.tail = FALSE, log.p = TRUE)
-            mat[i, ] = c(pval[i], stat[i]) 
-          }
-          stopCluster(cl)
-		}  
+		       cl <- makePSOCKcluster(ncores)
+           registerDoParallel(cl)
+           mat <- matrix(0, p, 2)
+           mod <- foreach( i = 1:p, .combine = rbind, .export = "speedlm", .packages = "speedglm" ) %dopar% {
+             ww = speedlm( target ~ dataset[, i], data = as.data.frame(dataset), weights = wei ) 
+			       suma = summary(ww)[[ 13 ]]
+             stat[i] = suma[1]
+             dof = suma[3]
+             pval[i] = pf(stat, 1, dof, lower.tail = FALSE, log.p = TRUE)
+             mat[i, ] = c(pval[i], stat[i]) 
+           }
+           stopCluster(cl)
+		     }  
 		
       } else {  ## Robust
-        fit0 = MASS::rlm(target ~ 1, maxit = 2000, weights = wei) 
+        fit0 = MASS::rlm( target ~ 1, maxit = 2000, weights = wei ) 
         stat0 = 2 * logLik(fit0)
         
          cl <- makePSOCKcluster(ncores)
@@ -134,12 +135,10 @@ lm.fsreg <- function(target, dataset,ini = NULL, threshold = 0.05, wei = NULL, s
          pval = pchisq(difa, mod[, 2] - 1, lower.tail = FALSE, log.p = TRUE)
          mod = cbind( pval, difa)
       }
-
       mat <- cbind(1:p, mod)      
-  
     }
     
-    colnames(mat) <- c( "variables", "p-value", "stat" )
+    colnames(mat) <- c( "variables", "log.p-value", "stat" )
     rownames(mat) <- 1:p
     sel <- which.min(mat[, 2])
     info <- matrix( numeric(3), ncol = 3 )
@@ -149,16 +148,15 @@ lm.fsreg <- function(target, dataset,ini = NULL, threshold = 0.05, wei = NULL, s
       info[1, ] <- mat[sel, ]
       mat <- mat[-sel, ] 
       if ( !is.matrix(mat) )   mat <- matrix(mat, ncol = 3) 
-      mat <- mat[ order( mat[, 2] ), ] 
 
       if ( stopping == "adjrsq" ) {
 
-        if ( robust == FALSE ) {
-		      if ( heavy == FALSE ) {
-            mi = lm( target ~ dataset[, sel], weights = wei )
+        if ( !robust ) {
+		      if ( !heavy ) {
+            mi = lm( target ~ dataset[, sel], weights = wei, y = FALSE, model = FALSE )
             tool[1] <- as.numeric( summary( mi )[[ 9 ]] )
 		      } else {
-            mi = speedglm::speedlm( target ~ dataset[, sel], data = as.data.frame(dataset), weights = wei )
+            mi = speedglm::speedlm( target ~ dataset[, sel], data = data.frame(dataset), weights = wei )
             tool[1] <- as.numeric( summary( mi )[[ 11 ]] )		  
 		      }
 		   
@@ -168,24 +166,26 @@ lm.fsreg <- function(target, dataset,ini = NULL, threshold = 0.05, wei = NULL, s
           tool[1] = 1 - (1 - r2) * (n - 1) / ( n - length( coef(mi) ) - 1 )
         }         
       } else if ( stopping  == "BIC" ) { 
-        if ( robust == FALSE ) {
-		      if ( heavy == FALSE ) {
-            mi = lm( target ~ dataset[, sel], weights = wei )
+        if ( !robust ) {
+		      if ( !heavy ) {
+            mi = lm( target ~ dataset[, sel], weights = wei, y = FALSE, model = FALSE )
             tool[1] <- BIC( mi )
 		      } else {
-            mi = speedglm::speedlm( target ~ dataset[, sel], data = as.data.frame(dataset), weights = wei )
-            tool[1] <-  - 2 * mi$logLik + length( coef(mi) ) * con
+            mi = speedglm::speedlm( target ~ dataset[, sel], data = data.frame(dataset), weights = wei )
+            tool[1] <-  BIC(mi)
 		      }	 
           
         } else {
           mi = MASS::rlm( target ~ dataset[, sel], maxit = 2000, weights = wei )
-          tool[1] <-  BIC(mi)
+          tool[1] <- BIC(mi)
         }
       }
       moda[[ 1 ]] <- mi
       
-    }
-    
+    }  else  {
+      info <- info  
+      sela <- NULL
+    }    
  
      ######
      ###### k equal to 2
@@ -198,10 +198,10 @@ lm.fsreg <- function(target, dataset,ini = NULL, threshold = 0.05, wei = NULL, s
       
       if ( ncores <= 1 ) {
 	  
-        if ( robust == FALSE ) {    
-          if ( heavy == FALSE ) {
+        if ( !robust ) {    
+          if ( !heavy ) {
            for (i in 1:pn) {
-             ww = lm( target ~ dataset[, sel] + dataset[, mat[i, 1] ], weights = wei )
+             ww = lm( target ~ dataset[, sel] + dataset[, mat[i, 1] ], weights = wei, y = FALSE, model = FALSE )
              tab = anova( ww )
              mat[i, 3] = tab[2, 4] 
              df1 = tab[2, 1]   ;  df2 = tab[3, 1]
@@ -220,9 +220,9 @@ lm.fsreg <- function(target, dataset,ini = NULL, threshold = 0.05, wei = NULL, s
               mat[i, 2] = pf( mat[i, 3], df1, df2, lower.tail = FALSE, log.p = TRUE )
             }  		  
 		  
-		      }
+		     }
 		  
-        } else {
+       } else {
           do = 2 * as.numeric( logLik( moda[[ 1 ]] ) )
           fr = length( coef( moda[[ 1 ]] ) )
           sta = dof = numeric(pn)
@@ -238,15 +238,15 @@ lm.fsreg <- function(target, dataset,ini = NULL, threshold = 0.05, wei = NULL, s
 
       } else {
 
-        if ( robust == FALSE ) {  ## Non robust
+        if ( !robust ) {  ## Non robust
 		
-		  if ( heavy == FALSE ) {
+		  if ( !heavy ) {
            cl <- makePSOCKcluster(ncores)
            registerDoParallel(cl)
            stat <- pval <- numeric(pn) 
            mata <- matrix(0, pn, 2)
            mod <- foreach( i = 1:pn, .combine = rbind) %dopar% {
-             ww <- lm( target ~ dataset[, sel] + dataset[, mat[i, 1] ], weights = wei )
+             ww <- lm( target ~ dataset[, sel] + dataset[, mat[i, 1] ], weights = wei, y = FALSE, model = FALSE )
              tab <- anova( ww )
              stat[i] <- tab[2, 4] 
              df1 <- tab[2, 1]   ;  df2 = tab[3, 1]
@@ -295,19 +295,17 @@ lm.fsreg <- function(target, dataset,ini = NULL, threshold = 0.05, wei = NULL, s
 
       }
 
-    }
-
       ina <- which.min(mat[, 2])
       sel <- mat[ina, 1]     
       
         if ( stopping == "adjrsq" ) {
           if ( mat[ina, 2] < threshold ) {
 		  
-            if ( robust == FALSE ) {
+            if ( !robust ) {
 			
-			        if ( heavy == FALSE ) {
-                ma = lm( target ~ dataset[, sela] + dataset[, sel], weights = wei )
-                tool[k] <- as.numeric( summary( ma )[[ 9 ]] )
+			        if ( !heavy ) {
+                 ma = lm( target ~ dataset[, sela] + dataset[, sel], weights = wei, y = FALSE, model = FALSE )
+                 tool[k] <- as.numeric( summary( ma )[[ 9 ]] )
 			        } else {
 			          ma = speedglm::speedlm( target ~ dataset[, sela] + dataset[, sel], data = as.data.frame(dataset), weights = wei )
                 tool[k] <- as.numeric( summary( ma )[[ 11 ]] )
@@ -326,29 +324,24 @@ lm.fsreg <- function(target, dataset,ini = NULL, threshold = 0.05, wei = NULL, s
               info <- rbind(info, mat[ina, ] )
               sela <- info[, 1]
               mat <- mat[-ina , ] 
-              if ( is.matrix(mat) ) {
-                mat <- mat[ order( mat[, 2] ), ]
-              }
-
+              if ( !is.matrix(mat) )   mat <- matrix(mat, ncol = 3) 
               moda[[ k ]] <- ma
             }
 
-          } else {
-           info <- rbind(info, c( 1e300, 0, 0 ) )
-          }
+          } else  info <- rbind(info, c( 1e300, 0, 0 ) )
 
         } else if ( stopping == "BIC" ) {
 		
           if ( mat[ina, 2] < threshold ) {
 		  
-            if ( robust == FALSE ) { 
-              if ( heavy == FALSE ) {			
-                ma = lm( target ~ dataset[, sela] + dataset[, sel], weights = wei )
+            if ( !robust ) { 
+              if ( !heavy ) {			
+                ma = lm( target ~ dataset[, sela] + dataset[, sel], weights = wei, y = FALSE, model = FALSE )
                 tool[2] <- BIC( ma )
-		      } else {
+		          } else {
                 ma = speedglm::speedlm( target ~ dataset[, sela] + dataset[, sel], data = as.data.frame(dataset), weights = wei )
-                tool[2] <-  - 2 * logLik(ma) + length( coef(ma) ) * con			  
-			  }
+                tool[2] <- BIC(ma)		  
+			        }
 			  
             } else {
               ma = MASS::rlm( target ~ dataset[, sela] + dataset[, sel], maxit = 2000, weights = wei )
@@ -362,25 +355,18 @@ lm.fsreg <- function(target, dataset,ini = NULL, threshold = 0.05, wei = NULL, s
               info <- rbind(info, mat[ina, ] )
               sela <- info[, 1]
               mat <- mat[-ina , ]
-              if ( !is.matrix(mat) ) {
-                mat <- matrix(mat, ncol = 3) 
-              }
-              mat <- mat[ order( mat[, 2] ), ]
-
+              if ( !is.matrix(mat) )   mat <- matrix(mat, ncol = 3) 
               moda[[ k ]] <- ma
             }
           } else  info <- info
           
         }
-
-
- 
- 
+  }
+    
      ######
      ###### k greater than 2
      ######
 
-	 
     if ( nrow(info) > 1  &  nrow(mat) > 0 ) {
       while ( ( info[k, 2] < threshold ) &  ( k < n ) & ( abs( tool[ k ] - tool[ k - 1 ] ) > tol ) & ( nrow(mat) > 0 ) )  {
          
@@ -389,20 +375,20 @@ lm.fsreg <- function(target, dataset,ini = NULL, threshold = 0.05, wei = NULL, s
         
         if ( ncores <= 1 ) {
 		
-          if ( robust == FALSE ) {
-		        if ( heavy == FALSE ) {
+          if ( !robust ) {
+		        if ( !heavy ) {
               for ( i in 1:pn ) {
-                 ww <- lm( target ~., data = as.data.frame( dataset[, c(sela, mat[i, 1]) ] ), weights = wei )
+                 ww <- lm( target ~., data = as.data.frame( dataset[, c(sela, mat[i, 1]) ] ), weights = wei, y = FALSE, model = FALSE )
                  tab <- anova( ww )
                  mat[i, 3] <- tab[k, 4] 
                  df1 <- tab[k, 1]   ;  df2 = tab[k + 1, 1]
                  mat[i, 2] <- pf( mat[i, 3], df1, df2, lower.tail = FALSE, log.p = TRUE )
               }
 			 
-			      } else {	
-		          fit1rss <- ma$RSS 
+		   } else {	
+		     fit1rss <- ma$RSS 
               d1 <- length( coef(ma) )
-		          for (i in 1:pn) {
+		      for (i in 1:pn) {
                 fit2 = speedglm::speedlm( target ~., data = as.data.frame( dataset[, c(sela, mat[i, 1]) ] ), weights= wei )
                 d2 = length( coef(fit2) )
                 df1 = d2 - d1
@@ -416,28 +402,27 @@ lm.fsreg <- function(target, dataset,ini = NULL, threshold = 0.05, wei = NULL, s
             
             do = 2 * as.numeric( logLik( moda[[ k - 1 ]] ) )
             fr = length( coef( moda[[ k - 1 ]] ) )
-            sta = dof = numeric(pn)
-            
-              for (i in 1:pn) {
-                ww = MASS::rlm( target ~., data = as.data.frame( dataset[, c(sela, mat[i, 1]) ] ), maxit = 2000, weights = wei )
-                sta[i] = 2 * as.numeric( logLik(ww) )
-                dof[i] = length( coef(ww) )
-              } 
-              mat[, 3] = abs( sta - do )
-              mat[, 2] = pchisq(mat[, 3], dof - fr, lower.tail = FALSE, log.p = TRUE)
-            }
+            sta = dof = numeric(pn)      
+            for (i in 1:pn) {
+              ww = MASS::rlm( target ~., data = as.data.frame( dataset[, c(sela, mat[i, 1]) ] ), maxit = 2000, weights = wei )
+              sta[i] = 2 * as.numeric( logLik(ww) )
+              dof[i] = length( coef(ww) )
+            } 
+            mat[, 3] = abs( sta - do )
+            mat[, 2] = pchisq(mat[, 3], dof - fr, lower.tail = FALSE, log.p = TRUE)
+          }
 
          } else {
 		
-          if ( robust == FALSE ) {  ## Non robust
+          if ( !robust ) {  ## Non robust
 		  
-		        if ( heavy == FALSE ) {
+		      if ( !heavy ) {
               cl <- makePSOCKcluster(ncores)
               registerDoParallel(cl)
               stat <- pval <- numeric(pn)
               mata <- matrix(0, pn, 2)
               mod <- foreach( i = 1:pn, .combine = rbind) %dopar% {
-                ww <- lm( target ~., data = as.data.frame( dataset[, c(sela, mat[ i, 1]) ] ), weights = wei )
+                ww <- lm( target ~., data = as.data.frame( dataset[, c(sela, mat[ i, 1]) ] ), weights = wei, y = FALSE, model = FALSE )
                 tab <- anova(ww)
                 stat[i] <- tab[k, 4] 
                 df1 <- tab[k, 1]   ;  df2 = tab[k + 1, 1]
@@ -494,14 +479,14 @@ lm.fsreg <- function(target, dataset,ini = NULL, threshold = 0.05, wei = NULL, s
 		  
             if ( mat[ina, 2] < threshold ) {
 			
-              if ( robust == FALSE ) {
-			    if ( heavy == FALSE ) {
-                  ma = lm( target ~., data = as.data.frame( dataset[, c(sela, sel)] ), weights = wei )
+              if ( !robust ) {
+			          if ( !heavy ) {
+                  ma = lm( target ~., data = as.data.frame( dataset[, c(sela, sel)] ), weights = wei, y = FALSE, model = FALSE )
                   tool[k] <- BIC( ma )
-				} else {
+				        } else {
                   ma = speedglm::speedlm( target ~., data = as.data.frame( dataset[, c(sela, sel)] ), weights = wei )
-                  tool[k] <-  - 2 * ma$logLik + length( coef(ma) ) * con			  
-				}
+                  tool[k] <-  BIC(ma)	  
+				        }
 				
               } else {
                 # ma = robust::lmRob( target ~., data = as.data.frame( dataset[, c(sela, sel)] ), control = cont )
@@ -516,11 +501,7 @@ lm.fsreg <- function(target, dataset,ini = NULL, threshold = 0.05, wei = NULL, s
                 info <- rbind( info, mat[ina, ] )
                 sela <- info[, 1]
                 mat <- mat[-ina , ] 
-                if ( !is.matrix(mat) ) {
-                  mat <- matrix(mat, ncol = 3) 
-                }
-                mat <- mat[ order( mat[, 2] ), ]
-
+                if ( !is.matrix(mat) )  mat <- matrix(mat, ncol = 3) 
                 moda[[ k ]] <- ma
               }
              
@@ -531,19 +512,19 @@ lm.fsreg <- function(target, dataset,ini = NULL, threshold = 0.05, wei = NULL, s
           } else if ( stopping == "adjrsq" ) {
             if ( mat[ina, 2] < threshold ) {
 
-              if ( robust == FALSE ) {
-			    if ( heavy == FALSE ) {
-                  ma = lm( target ~., data = as.data.frame( dataset[, c(sela, sel)] ), weights = wei )
+              if ( !robust ) {
+			          if ( !heavy ) {
+                  ma = lm( target ~., data = as.data.frame( dataset[, c(sela, sel)] ), weights = wei, y = FALSE, model = FALSE )
                   tool[k] <- as.numeric( summary(ma)[[ 9 ]] )
-				} else {
-				  ma = speedglm::speedlm( target ~., data = as.data.frame( dataset[, c(sela, sel)] ), weights = wei )
+				        } else {
+				          ma = speedglm::speedlm( target ~., data = as.data.frame( dataset[, c(sela, sel)] ), weights = wei )
                   tool[k] <- as.numeric( summary(ma)[[ 11 ]] )
-				}
+				        }
 				
               } else {
                 ma = MASS::rlm( target ~., data = as.data.frame( dataset[, c(sela, sel)] ), maxit = 2000, weights = wei )
                 r2 = cor(target, fitted(ma) )^2
-                tool[k] = 1 - (1 - r2) * (n - 1) / ( n - length( coef(ma) ) - 1)
+                tool[k] <- 1 - (1 - r2) * (n - 1) / ( n - length( coef(ma) ) - 1)
               }               
               if ( tool[ k ] - tool[ k - 1 ] <= tol ) {
                 info <- rbind(info, c( 1e300, 0, 0 ) )
@@ -552,16 +533,11 @@ lm.fsreg <- function(target, dataset,ini = NULL, threshold = 0.05, wei = NULL, s
                 info <- rbind( info, mat[ina, ] )
                 sela <- info[, 1]
                 mat <- mat[-ina , ]
-                if ( is.matrix(mat) ) {
-                  mat <- mat[ order( mat[, 2] ), ]
-                }
-
+                if ( !is.matrix(mat) )   mat <- matrix(mat, ncol = 3) 
                 moda[[ k ]] <- ma
               }
 
-            } else {
-              info <- rbind(info, c( 1e300, 0, 0 ) )
-            }
+            } else  info <- rbind(info, c( 1e300, 0, 0 ) )
         
           }
       }
@@ -584,43 +560,34 @@ lm.fsreg <- function(target, dataset,ini = NULL, threshold = 0.05, wei = NULL, s
         xx <- as.data.frame( dataset[, sela] )
         colnames(xx) <- paste("V", sela, sep = "") 
 
-          if ( robust == FALSE ) {
-		    if ( heavy == FALSE ) {
-              models[[ 1 ]] <- final <- lm( target ~., data = as.data.frame( xx ), weights = wei )
-			} else { 
-              models[[ 1 ]] <- final <- speedglm::speedlm( target ~., data = as.data.frame( xx ), weights = wei )
-            } 
-		  } else {
-            models[[ 1 ]] <- final <- MASS::rlm( target ~., data = as.data.frame( xx ), maxit = 2000, weights = wei )
-          }
+       if ( !robust ) {
+		     if ( !heavy ) {
+              models[[ 1 ]] <- final <- lm( target ~., data = as.data.frame( xx ), weights = wei, y = FALSE, model = FALSE )
+			   } else  models[[ 1 ]] <- final <- speedglm::speedlm( target ~., data = as.data.frame( xx ), weights = wei )
+			   
+		   } else  models[[ 1 ]] <- final <- MASS::rlm( target ~., data = as.data.frame( xx ), maxit = 2000, weights = wei )
         
       } else {
         for (i in 1:d) {
 		
-          if ( robust == FALSE ) {
-		    if ( heavy == FALSE ) {
-              models[[ i ]] <- lm( target ~., data = as.data.frame( xx[, 1:i] ), weights = wei )
-			} else {  
-              models[[ i ]] <- speedglm::speedlm( target ~., data = as.data.frame( xx[, 1:i] ), weights = wei )
-            }
+          if ( !robust ) {
+		        if ( !heavy ) {
+              models[[ i ]] <- lm( target ~., data = as.data.frame( xx[, 1:i] ), weights = wei, y = FALSE, model = FALSE )
+			      } else   models[[ i ]] <- speedglm::speedlm( target ~., data = as.data.frame( xx[, 1:i] ), weights = wei )
 			
-			} else { 
-            models[[ i ]] <- MASS::rlm( target ~., data = data.frame( xx[, 1:i]), maxit = 2000, weights = wei )   
-          }
-		  
+		      } else   models[[ i ]] <- MASS::rlm( target ~., data = data.frame( xx[, 1:i]), maxit = 2000, weights = wei )  
         }
       }
 
       final <- models[[ d ]]
-
       info <- info[1:d, ]
       if ( d == 1 )  info <- matrix(info, nrow = 1)
       info <- cbind( info, tool[ 1:d ] ) 
-      colnames(info) <- c( "variables", "p-value", "stat", stopping )
+      colnames(info) <- c( "variables", "log.p-value", "stat", stopping )
       rownames(info) <- info[, 1]
     }
 
-    result <- list(mat = t(mat), info = info, models = models, final = final, ci_test = ci_test, runtime = runtime ) 
+    result <- list(runtime = runtime, mat = t(mat), info = info, models = models, ci_test = ci_test, final = final ) 
     
   }  
   
