@@ -6,36 +6,33 @@ lm.fsreg_2 <- function(target, dataset, iniset = NULL, threshold = 0.05, wei = N
   pval <- stat <- dof <- numeric( p )  
   moda <- list()
   
-    #check for NA values in the dataset and replace them with the variable median or the mode
-  if(any(is.na(dataset)) == TRUE)
-  {
-
+  ## percentages
+  if ( min( target ) > 0  &  max( target ) < 1 )  {  ## are they percentages?
+    target <- log( target / (1 - target) ) 
+  }
   
+    #check for NA values in the dataset and replace them with the variable median or the mode
+  if( any(is.na(dataset)) ) {
     #dataset = as.matrix(dataset);
     warning("The dataset contains missing values (NA) and they were replaced automatically by the variable (column) median (for numeric) or by the most frequent level (mode) if the variable is factor")
-    
     if (class(dataset) == "matrix")  {
-    
-       dataset = apply( dataset, 2, function(x){ x[which(is.na(x))] = median(x, na.rm = TRUE) ; return(x)} ) 
-              
+      dataset <- apply( dataset, 2, function(x){ x[which(is.na(x))] = median(x, na.rm = TRUE) ; return(x) } ) 
     }else{
-	
-    poia <- which( is.na(dataset), arr.ind = TRUE )[2]
- 	for( i in poia )
-      {
-          xi = dataset[, i]
-          if(class(xi) == "numeric")
-          {                    
-            xi[ which( is.na(xi) ) ] = median(xi, na.rm = TRUE) 
-          } else if ( class(xi) == "factor" ) {
-            xi[ which( is.na(xi) ) ] = levels(xi)[ which.max( as.vector( table(xi) ) )]
-          }
-          dataset[, i] = xi
+      poia <- which( is.na(dataset), arr.ind = TRUE )[2]
+      for( i in poia )  {
+        xi <- dataset[, i]
+        if(class(xi) == "numeric")
+        {                    
+          xi[ which( is.na(xi) ) ] <- median(xi, na.rm = TRUE) 
+        } else if ( is.factor( xi ) ) {
+          xi[ which( is.na(xi) ) ] <- levels(xi)[ which.max( as.vector( table(xi) ) )]
         }
+        dataset[, i] <- xi
+      }
     }
-    
   }
   
+  
   ## is there already an initial set of variables to start with?
   if ( is.null(iniset) ) {
     da <- 0
@@ -59,16 +56,16 @@ lm.fsreg_2 <- function(target, dataset, iniset = NULL, threshold = 0.05, wei = N
   
   if (ncores <= 1) {
     
-    if ( robust == FALSE ) {  ## Non robust
+    if ( !robust ) {  ## Non robust
 	
-	  if ( heavy == FALSE ) {
-	      ci_test <- "testsIndReg"
+	  if ( !heavy ) {
+	    ci_test <- "testsIndReg"
         for (i in 1:p) {
-          ww = lm( target ~., data = as.data.frame( dataset[, c(da, pa + i)] ), weights = wei )
-          tab = anova(ww)
-          stat[i] = tab[pa + k, 4] 
-          df1 = tab[pa + k, 1]   ;  df2 = tab[pa + k + 1, 1]
-          pval[i] = pf( stat[i], df1, df2, lower.tail = FALSE, log.p = TRUE )
+          ww <- lm( target ~., data = as.data.frame( dataset[, c(da, pa + i)] ), weights = wei, y = FALSE, model = FALSE )
+          tab <- anova(ww)
+          stat[i] <- tab[pa + k, 4] 
+          df1 <- tab[pa + k, 1]   ;  df2 = tab[pa + k + 1, 1]
+          pval[i] <- pf( stat[i], df1, df2, lower.tail = FALSE, log.p = TRUE )
         } 
 		
 	  } else {
@@ -77,13 +74,13 @@ lm.fsreg_2 <- function(target, dataset, iniset = NULL, threshold = 0.05, wei = N
 		  fit1rss <- fit1$RSS 
       d1 <- length( coef(fit1) )
 		  for (i in 1:pn) {
-        fit2 = speedglm::speedlm( target ~., data = as.data.frame( dataset[, c(da, pa + i)] ), weights= wei )
-        d2 = length( coef(fit2) )
-        df1 = d2 - d1
-        df2 = n - d2
-        stat[i] = (fit1rss - fit2$RSS)/df1 / ( fit2$RSS /df2 )
-        pval[i] = pf( stat[i], df1, df2, lower.tail = FALSE, log.p = TRUE )
-      } 	
+          fit2 <- speedglm::speedlm( target ~., data = as.data.frame( dataset[, c(da, pa + i)] ), weights= wei )
+          d2 <- length( coef(fit2) )
+          df1 <- d2 - d1
+          df2 <- n - d2
+          stat[i] <- (fit1rss - fit2$RSS)/df1 / ( fit2$RSS /df2 )
+          pval[i] <- pf( stat[i], df1, df2, lower.tail = FALSE, log.p = TRUE )
+        } 	
 	  
 	  }	
       mat <- cbind(1:p, pval, stat)
@@ -91,28 +88,28 @@ lm.fsreg_2 <- function(target, dataset, iniset = NULL, threshold = 0.05, wei = N
     } else {  ## Robust
       
       for (i in 1:p) {
-        ww = MASS::rlm( target ~., data = as.data.frame( dataset[, c(da, pa + i)] ), maxit = 2000, weights = wei ) 
-        stat[i] = 2 * as.numeric( logLik(ww) )
-        dof[i] = length( coef(ww) )
+        ww <- MASS::rlm( target ~., data = as.data.frame( dataset[, c(da, pa + i)] ), maxit = 2000, weights = wei ) 
+        stat[i] <- 2 * as.numeric( logLik(ww) )
+        dof[i] <- length( coef(ww) )
       }
 	  
-      fit0 = MASS::rlm(target ~ ., data = as.data.frame(iniset), maxit = 2000, weights = wei) 
-      stat0 = 2 * as.numeric( logLik(fit0) )
-      difa = abs( stat - stat0 )
-      pval = pchisq(difa, dof - 1, lower.tail = FALSE, log.p = TRUE)
+      fit0 <- MASS::rlm( target ~ ., data = as.data.frame(iniset), maxit = 2000, weights = wei ) 
+      stat0 <- 2 * as.numeric( logLik(fit0) )
+      difa <- abs( stat - stat0 )
+      pval <- pchisq(difa, dof - 1, lower.tail = FALSE, log.p = TRUE)
       mat <- cbind(1:p, pval, difa)
     }
     
   } else {
     
-    if ( robust == FALSE ) {  ## Non robust
+    if ( !robust ) {  ## Non robust
 	
-	  if ( heavy == FALSE ) {
+	  if ( !heavy ) {
         cl <- makePSOCKcluster(ncores)
         registerDoParallel(cl)
         mat <- matrix(0, p, 2)
         mod <- foreach( i = 1:p, .combine = rbind ) %dopar% {
-          ww <- lm( target ~., data = as.data.frame( dataset[, c(da, pa + i)] ), weights = wei )
+          ww <- lm( target ~., data = as.data.frame( dataset[, c(da, pa + i)] ), weights = wei, y = FALSE, model = FALSE )
           tab <- anova( ww )
           stat[i] <- tab[pa + k, 4] 
           df1 <- tab[pa + k, 1]   ;  df2 = tab[pa + k + 1, 1]
@@ -131,11 +128,11 @@ lm.fsreg_2 <- function(target, dataset, iniset = NULL, threshold = 0.05, wei = N
         mat <- matrix(0, p, 2)
         mod <- foreach( i = 1:p, .combine = rbind, .export = "speedlm", .packages = "speedglm" ) %dopar% {
           fit2 <- speedlm( target ~., data = as.data.frame( dataset[, c(da, pa + i)] ), weights = wei )
-          d2 = length( coef(fit2) )
-          df1 = d2 - d1
-          df2 = n - d2
-          stat[i] = (fit1rss - fit2$RSS)/df1 / ( fit2$RSS /df2 )
-          pval[i] = pf( stat[i], df1, df2, lower.tail = FALSE, log.p = TRUE )
+          d2 <- length( coef(fit2) )
+          df1 <- d2 - d1
+          df2 <- n - d2
+          stat[i] <- (fit1rss - fit2$RSS)/df1 / ( fit2$RSS /df2 )
+          pval[i] <- pf( stat[i], df1, df2, lower.tail = FALSE, log.p = TRUE )
           mat[i, ] <- c(pval[i], stat[i]) 
         }
 		
@@ -144,7 +141,7 @@ lm.fsreg_2 <- function(target, dataset, iniset = NULL, threshold = 0.05, wei = N
 	       
     } else {  ## Robust
       
-      fit0 = MASS::rlm(target ~., data = as.data.frame(iniset), maxit = 2000, weights = wei) 
+      fit0 = MASS::rlm( target ~., data = as.data.frame(iniset), maxit = 2000, weights = wei ) 
       stat0 = 2 * logLik(fit0)
       
       cl <- makePSOCKcluster(ncores)
@@ -165,7 +162,7 @@ lm.fsreg_2 <- function(target, dataset, iniset = NULL, threshold = 0.05, wei = N
     
   }
   
-  colnames(mat) <- c( "variables", "p-value", "stat" )
+  colnames(mat) <- c( "variables", "log.p-value", "stat" )
   rownames(mat) <- 1:p
   sel <- which.min(mat[, 2])
   sela <- sel
@@ -174,22 +171,19 @@ lm.fsreg_2 <- function(target, dataset, iniset = NULL, threshold = 0.05, wei = N
     
     info[k, ] <- mat[sel, ]
     mat <- mat[-sel, ] 
-    if ( !is.matrix(mat) ) {
-      mat <- matrix(mat, ncol = 3) 
-    }
-    mat <- mat[ order( mat[, 2] ), ] 
+    if ( !is.matrix(mat) )   mat <- matrix(mat, ncol = 3) 
     
     if ( stopping == "adjrsq" ) {
       
-      if ( robust == FALSE ) {
+      if ( !robust ) {
 	  
-	      if ( heavy == TRUE ) {
-          mi = lm( target ~., data = as.data.frame( dataset[, c(da, sel) ] ), weights = wei )
+	    if ( !heavy ) {
+          mi = lm( target ~., data = as.data.frame( dataset[, c(da, sel) ] ), weights = wei, y = FALSE, model = FALSE )
           tool[k] <- as.numeric( summary( mi )[[ 9 ]] )
-		    } else {
-		      mi = speedglm::speedlm( target ~., data = as.data.frame( dataset[, c(da, sel) ] ), weights = wei )
+		} else {
+		  mi = speedglm::speedlm( target ~., data = as.data.frame( dataset[, c(da, sel) ] ), weights = wei )
           tool[k] <- as.numeric( summary( mi )[[ 11 ]] )
-		    }
+		}
 		
       } else {
         mi = MASS::rlm( target ~., data = as.data.frame( dataset[, c(da, sel) ] ), maxit = 2000, weights = wei )
@@ -198,14 +192,14 @@ lm.fsreg_2 <- function(target, dataset, iniset = NULL, threshold = 0.05, wei = N
       } 
       
     } else if ( stopping  == "BIC" ) { 
-      if ( robust == FALSE ) {
-	      if ( heavy == FALSE ) {
-          mi = lm( target ~., data = as.data.frame( dataset[, c(da, sel) ] ) )
+      if ( !robust ) {
+	    if ( !heavy ) {
+          mi = lm( target ~., data = as.data.frame( dataset[, c(da, sel) ] ), y = FALSE, model = FALSE )
           tool[k] <- BIC( mi )
-		    } else {
+		} else {
           mi = speedglm::speedlm( target ~., data = as.data.frame( dataset[, c(da, sel) ] ), weights = wei )
-          tool[k] <-  - 2 * mi$logLik + length( coef(mi) ) * con 
-	   	  }
+          tool[k] <-  - 2 * logLik(mi) + length( coef(mi) ) * con 
+	   	}
 		
       } else {
         mi = MASS::rlm( target ~., data=as.data.frame( dataset[, c(da, sel) ] ), maxit = 2000, weights = wei )
@@ -228,11 +222,11 @@ lm.fsreg_2 <- function(target, dataset, iniset = NULL, threshold = 0.05, wei = N
     
     if ( ncores <= 1 ) {
 	
-      if ( robust == FALSE ) { 
+      if ( !robust ) { 
 	  
-        if ( heavy == FALSE ) {
+        if ( !heavy ) {
           for (i in 1:pn) {
-            ww = lm( target ~., data = as.data.frame( dataset[, c(da, sel, mat[pa + i, 1]) ] ), weights = wei )
+            ww = lm( target ~., data = as.data.frame( dataset[, c(da, sel, mat[pa + i, 1]) ] ), weights = wei, y = FALSE, model = FALSE )
             tab = anova( ww )
             mat[i, 3] = tab[pa + k, 4] 
             df1 = tab[pa + k, 1]   ;  df2 = tab[pa + k + 1, 1]
@@ -252,13 +246,13 @@ lm.fsreg_2 <- function(target, dataset, iniset = NULL, threshold = 0.05, wei = N
 		     }	  
 		   }
 		
-      } else {
+     } else {
         do = 2 * as.numeric( logLik( mi ) )
         fr = length( coef( mi ) )
         sta = dof = numeric(pn)
         
         for (i in 1:pn) {
-          ww = MASS::rlm( target ~., data = as.data.frame( dataset[, c(da, sel, mat[pa + i, 1]) ] ), maxit = 2000, weights = wei )
+          ww = MASS::rlm( target ~., data = as.data.frame( dataset[, c(da, sel, mat[pa + i, 1]) ] ), maxit = 2000, weights = wei)
           sta[i] = 2 * as.numeric( logLik(ww) )
           dof[i] = length( coef(ww) )
         } 
@@ -268,57 +262,57 @@ lm.fsreg_2 <- function(target, dataset, iniset = NULL, threshold = 0.05, wei = N
       
     } else {
       
-      if ( robust == FALSE ) {  ## Non robust
+      if ( !robust ) {  ## Non robust
 	  
-	      if ( heavy == FALSE ) {
+	    if ( !heavy ) {
           cl <- makePSOCKcluster(ncores)
           registerDoParallel(cl)
           stat <- pval <- numeric(pn) 
           mata <- matrix(0, pn, 2)
           mod <- foreach( i = 1:pn, .combine = rbind) %dopar% {
-            ww <- lm( target ~., data = as.data.frame( dataset[, c(da, sel, mat[pa + i, 1]) ] ), weights = wei )
+            ww <- lm( target ~., data = as.data.frame( dataset[, c(da, sel, mat[pa + i, 1]) ] ), weights = wei, y = FALSE, model = FALSE )
             tab <- anova( ww )
             stat[i] <- tab[pa + k, 4] 
             df1 <- tab[pa + k, 1]   ;  df2 = tab[pa + k + 1, 1]
             pval[i] <- pf( stat[i], df1, df2, lower.tail = FALSE, log.p = TRUE )
             mata[i, ] <- c(pval[i], stat[i]) 
-          }
+        }
           stopCluster(cl)
 		  
         } else {
-		      cl <- makePSOCKcluster(ncores)
+		  cl <- makePSOCKcluster(ncores)
           registerDoParallel(cl)
           fit1rss <- mi$RSS 
           stat <- pval <- numeric(pn) 
           mata <- matrix(0, pn, 2)
           mod <- foreach( i = 1:pn, .combine = rbind, .export = "speedlm", .packages = "speedglm") %dopar% {
             fit2 <- speedlm( target ~., data = as.data.frame( dataset[, c(da, sel, mat[pa + i, 1]) ] ), weights = wei )
-            d2 = length( coef(fit2) )
-            df1 = d2 - d1
-            df2 = n - d2
-            stat[i] = (fit1rss - fit2$RSS)/df1 / ( fit2$RSS /df2 )
-            pval[i] = pf( mat[i, 3], df1, df2, lower.tail = FALSE, log.p = TRUE )
+            d2 <- length( coef(fit2) )
+            df1 <- d2 - d1
+            df2 <- n - d2
+            stat[i] <- (fit1rss - fit2$RSS)/df1 / ( fit2$RSS /df2 )
+            pval[i] <- pf( mat[i, 3], df1, df2, lower.tail = FALSE, log.p = TRUE )
             mata[i, ] <- c( pval[i], stat[i] ) 
           }
           stopCluster(cl)
 		   }
 		
       } else {  ## Robust
-        do = 2 * as.numeric( logLik( mi ) )
-        fr = length( coef( mi ) )
+        do <- 2 * as.numeric( logLik( mi ) )
+        fr <- length( coef( mi ) )
         
         cl <- makePSOCKcluster(ncores)
         registerDoParallel(cl)
         mata <- matrix(0, pn, 2)
         mod <- foreach( i = 1:pn, .combine = rbind, .export = c("rlm"), .packages = "MASS" ) %dopar% {
-          ww <- rlm( target ~., data = as.data.frame( dataset[, c(da, sel, mat[pa + i, 1]) ] ), maxit = 2000, weights = wei )
-          mata[i, ] = c( 2 * as.numeric( logLik(ww) ), length( coef(ww) ) )
+          ww <- rlm( target ~., data = as.data.frame( dataset[, c(da, sel, mat[pa + i, 1]) ] ), maxit = 2000, weights = wei)
+          mata[i, ] <- c( 2 * as.numeric( logLik(ww) ), length( coef(ww) ) )
         }
         stopCluster(cl)
         
-        difa = abs( mod[, 1] - do )
-        pval = pchisq(difa, mod[, 2] - fr, lower.tail = FALSE, log.p = TRUE)
-        mod = cbind( pval, difa)
+        difa <- abs( mod[, 1] - do )
+        pval <- pchisq(difa, mod[, 2] - fr, lower.tail = FALSE, log.p = TRUE)
+        mod <- cbind( pval, difa)
       } 
       mat <- cbind(mat[, 1], mod)   
       
@@ -332,19 +326,19 @@ lm.fsreg_2 <- function(target, dataset, iniset = NULL, threshold = 0.05, wei = N
   if ( stopping == "adjrsq" ) {
   
     if ( mat[ina, 2] < threshold ) {
-      if ( robust == FALSE ) {
-	    if ( heavy == FALSE ) {
-          ma = lm( target ~., data=as.data.frame( dataset[, c(da, sela, sel) ] ), weights = wei )
+      if ( !robust ) {
+	      if ( !heavy ) {
+          ma <- lm( target ~., data=as.data.frame( dataset[, c(da, sela, sel) ] ), weights = wei, y = FALSE, model = FALSE )
           tool[k] <- as.numeric( summary( ma )[[ 9 ]] )
-		} else {
-          ma = speedglm::speedlm( target ~., data=as.data.frame( dataset[, c(da, sela, sel) ] ), weights = wei )
+		    } else {
+          ma <- speedglm::speedlm( target ~., data=as.data.frame( dataset[, c(da, sela, sel) ] ), weights = wei, y = FALSE, model = FALSE )
           tool[k] <- as.numeric( summary( ma )[[ 11 ]] )		
-		}
+		    }
 		
       } else {        
-        ma= MASS::rlm( target ~., data=as.data.frame( dataset[, c(da, sela, sel) ] ), maxit = 2000, weights = wei )
-        r2 = cor( target, fitted(ma) )^2
-        tool[k] = 1 - (1 - r2) * (n - 1) / ( n - length( coef(ma) ) - 1)
+        ma <- MASS::rlm( target ~., data=as.data.frame( dataset[, c(da, sela, sel) ] ), maxit = 2000, weights = wei )
+        r2 <- cor( target, fitted(ma) )^2
+        tool[k] <- 1 - (1 - r2) * (n - 1) / ( n - length( coef(ma) ) - 1)
       }
       
       if ( tool[ k ] - tool[ k - 1 ] <= tol ) {
@@ -354,10 +348,7 @@ lm.fsreg_2 <- function(target, dataset, iniset = NULL, threshold = 0.05, wei = N
         info <- rbind(info, mat[ina, ] )
         sela <- info[, 1]
         mat <- mat[-ina , ] 
-        if ( is.matrix(mat) ) {
-          mat <- mat[ order( mat[, 2] ), ]
-        }
-        
+        if ( !is.matrix(mat) )   mat <- matrix(mat, ncol = 3) 
         moda[[ k ]] <- ma
       }
       
@@ -366,18 +357,18 @@ lm.fsreg_2 <- function(target, dataset, iniset = NULL, threshold = 0.05, wei = N
   } else if ( stopping == "BIC" ) {
   
     if ( mat[ina, 2] < threshold ) {
-      if ( robust == FALSE ) {            
-	    if ( heavy == FALSE ) {
-          ma = lm( target ~., data = as.data.frame( dataset[, c(da, sela, sel) ] ), weights = wei )
+      if ( !robust ) {            
+	      if ( !heavy ) {
+          ma <- lm( target ~., data = as.data.frame( dataset[, c(da, sela, sel) ] ), weights = wei, y = FALSE, model = FALSE )
           tool[k] <- BIC( ma )
-		} else {
-          ma = speedglm::speedlm( target ~., data = as.data.frame( dataset[, c(da, sela, sel) ] ), weights = wei )
-          tool[k] <-  - 2 *ma$logLik + length( coef(ma ) ) * con
-		}
+		    } else {
+          ma <- speedglm::speedlm( target ~., data = as.data.frame( dataset[, c(da, sela, sel) ] ), weights = wei )
+          tool[k] <-  - 2 * logLik(ma) + length( coef(ma ) ) * con
+	    	}
 		
       } else {
-        ma = MASS::rlm(target ~ target ~., data = as.data.frame( dataset[, c(da, sela, sel) ] ), maxit = 2000, weights = wei )
-        tool[k] = BIC(ma)
+        ma <- MASS::rlm(target ~ target ~., data = as.data.frame( dataset[, c(da, sela, sel) ] ), maxit = 2000, weights = wei )
+        tool[k] <- BIC(ma)
       }
       if ( tool[ k - 1] - tool[ k ] <= tol ) {
         info <- rbind(info, c( 1e300, 0, 0 ) )
@@ -386,11 +377,7 @@ lm.fsreg_2 <- function(target, dataset, iniset = NULL, threshold = 0.05, wei = N
         info <- rbind(info, mat[ina, ] )
         sela <- info[, 1]
         mat <- mat[-ina , ]
-        if ( !is.matrix(mat) ) {
-          mat <- matrix(mat, ncol = 3) 
-        }
-        mat <- mat[ order( mat[, 2] ), ]
-        
+        if ( !is.matrix(mat) )  mat <- matrix(mat, ncol = 3) 
         moda[[ k ]] <- ma
       }
     } else  info <- info
@@ -413,11 +400,11 @@ lm.fsreg_2 <- function(target, dataset, iniset = NULL, threshold = 0.05, wei = N
       
       if ( ncores <= 1 ) {
 	  
-        if ( robust == FALSE ) {
+        if ( !robust ) {
 		
-		      if ( heavy == FALSE ) {
+		  if ( !heavy ) {
             for ( i in 1:pn ) {
-              ww <- lm( target ~., data = as.data.frame( dataset[, c(da, sela, mat[pa + i, 1]) ] ), weights = wei )
+              ww <- lm( target ~., data = as.data.frame( dataset[, c(da, sela, mat[pa + i, 1]) ] ), weights = wei, y = FALSE, model = FALSE )
               tab <- anova( ww )
               mat[i, 3] <- tab[pa + k, 4] 
               df1 <- tab[pa + k, 1]   ;  df2 = tab[pa + k + 1, 1]
@@ -425,17 +412,17 @@ lm.fsreg_2 <- function(target, dataset, iniset = NULL, threshold = 0.05, wei = N
             }
 			
           } else {
-		        fit1rss <- ma$RSS 
+		    fit1rss <- ma$RSS 
             d1 <- length( coef(fit1) )
-		        for (i in 1:pn) {
+		    for (i in 1:pn) {
               fit2 = speedglm::speedlm( target ~., data = as.data.frame( dataset[, c(da, sel, mat[pa + i, 1]) ] ), weights= wei )
               d2 = length( coef(fit2) )
               df1 = d2 - d1
               df2 = n - d2
               mat[i, 3] = (fit1rss - fit2$RSS)/df1 / ( fit2$RSS /df2 )
               mat[i, 2] = pf( mat[i, 3], df1, df2, lower.tail = FALSE, log.p = TRUE )
-		        }			  
-		      }
+		    }			  
+		  }
 		  
         } else {
           
@@ -453,15 +440,15 @@ lm.fsreg_2 <- function(target, dataset, iniset = NULL, threshold = 0.05, wei = N
         }
         
       } else {
-        if ( robust == FALSE ) {  ## Non robust
+        if ( !robust ) {  ## Non robust
 		
-		     if ( heavy == FALSE ) {
+		  if ( !heavy ) {
             cl <- makePSOCKcluster(ncores)
             registerDoParallel(cl)
             stat <- pval <- numeric(pn)
             mata <- matrix(0, pn, 2)
             mod <- foreach( i = 1:pn, .combine = rbind) %dopar% {
-              ww <- lm( target ~., data = as.data.frame( dataset[, c(da, sela, mat[pa + i, 1]) ] ), weights = wei )
+              ww <- lm( target ~., data = as.data.frame( dataset[, c(da, sela, mat[pa + i, 1]) ] ), weights = wei, y = FALSE, model = FALSE )
               tab <- anova(ww)
               stat[i] <- tab[pa + k, 4] 
               df1 <- tab[pa + k, 1]   ;  df2 = tab[pa + k + 1, 1]
@@ -470,23 +457,23 @@ lm.fsreg_2 <- function(target, dataset, iniset = NULL, threshold = 0.05, wei = N
             }
             stopCluster(cl)
 		  
-		    } else {
-		      cl <- makePSOCKcluster(ncores)
-          registerDoParallel(cl)
-          fit1rss <- ma$RSS 
-          stat <- pval <- numeric(pn) 
-          mata <- matrix(0, pn, 2)
-          mod <- foreach( i = 1:pn, .combine = rbind, .export = "speedlm", .packages = "speedglm") %dopar% {
-            fit2 <- speedlm( target ~., data = as.data.frame( dataset[, c(da, sela, mat[pa + i, 1]) ] ), weights = wei )
-            d2 = length( coef(fit2) )
-            df1 = d2 - d1
-            df2 = n - d2
-            stat[i] = (fit1rss - fit2$RSS)/df1 / ( fit2$RSS /df2 )
-            pval[i] = pf( mat[i, 3], df1, df2, lower.tail = FALSE, log.p = TRUE )
-            mata[i, ] <- c( pval[i], stat[i] ) 
-          }
-          stopCluster(cl)
-		    }
+		  } else {
+		    cl <- makePSOCKcluster(ncores)
+            registerDoParallel(cl)
+            fit1rss <- ma$RSS 
+            stat <- pval <- numeric(pn) 
+            mata <- matrix(0, pn, 2)
+            mod <- foreach( i = 1:pn, .combine = rbind, .export = "speedlm", .packages = "speedglm") %dopar% {
+              fit2 <- speedlm( target ~., data = as.data.frame( dataset[, c(da, sela, mat[pa + i, 1]) ] ), weights = wei )
+              d2 = length( coef(fit2) )
+              df1 = d2 - d1
+              df2 = n - d2
+              stat[i] = (fit1rss - fit2$RSS)/df1 / ( fit2$RSS /df2 )
+              pval[i] = pf( mat[i, 3], df1, df2, lower.tail = FALSE, log.p = TRUE )
+              mata[i, ] <- c( pval[i], stat[i] ) 
+            }
+            stopCluster(cl)
+		  }
           
        } else {  ## Robust
           do = 2 * as.numeric( logLik( ma) )
@@ -516,19 +503,18 @@ lm.fsreg_2 <- function(target, dataset, iniset = NULL, threshold = 0.05, wei = N
 	  
         if ( mat[ina, 2] < threshold ) {
 		
-          if ( robust == FALSE ) {
-		        if ( heavy == FALSE ){
-              ma <- lm( target ~., data = as.data.frame( dataset[, c(da, sela, sel)] ), weights = wei )
+          if ( !robust ) {
+		    if ( !heavy ){
+              ma <- lm( target ~., data = as.data.frame( dataset[, c(da, sela, sel)] ), weights = wei, y = FALSE, model = FALSE )
               tool[k] <- BIC( ma )
-			      } else {
+			} else {
               ma <- speedglm::speedlm( target ~., data = as.data.frame( dataset[, c(da, sela, sel)] ), weights = wei )
-              tool[k] <-  - 2 *ma$logLik + length( coef(ma ) ) * con		
-			      }  
+              tool[k] <-  - 2 * logLik(ma) + length( coef(ma ) ) * con		
+			}  
 			
           } else {
-            # ma = robust::lmRob( target ~., data = as.data.frame( dataset[, c(sela, sel)] ), control = cont )
-            ma = MASS::rlm( target ~., data = as.data.frame( dataset[, c(da, sela, sel)] ), maxit = 2000, weights = wei )
-            tool[k] =  BIC(ma)
+            ma <- MASS::rlm( target ~., data = as.data.frame( dataset[, c(da, sela, sel)] ), maxit = 2000, weights = wei )
+            tool[k] <-  BIC(ma)
           }
           
           if ( tool[ k - 1] - tool[ k ] <= tol ) {
@@ -538,35 +524,29 @@ lm.fsreg_2 <- function(target, dataset, iniset = NULL, threshold = 0.05, wei = N
             info <- rbind( info, mat[ina, ] )
             sela <- info[, 1]
             mat <- mat[-ina , ] 
-            if ( !is.matrix(mat) ) {
-              mat <- matrix(mat, ncol = 3) 
-            }
-            mat <- mat[ order( mat[, 2] ), ]
-            
+            if ( !is.matrix(mat) )  mat <- matrix(mat, ncol = 3) 
             moda[[ k ]] <- ma
           }
           
-        } else {
-          info <- rbind(info, c( 1e300, 0, 0 ) )
-        }
+        } else  info <- rbind(info, c( 1e300, 0, 0 ) )
         
       } else if ( stopping == "adjrsq" ) {
 	    if ( mat[ina, 2] < threshold ) {
           
-          if ( robust == FALSE ) {
-		    if ( heavy == FALSE ) {
-              ma <- lm( target ~., data = as.data.frame( dataset[, c(da, sela, sel)] ), weights = wei )
-              tool[k] <- as.numeric( summary(ma)[[ 9 ]] )
-			} else {
-			  ma <- speedglm::speedlm( target ~., data=as.data.frame( dataset[, c(da, sela, sel) ] ), weights = wei )
-              tool[k] <- as.numeric( summary( ma )[[ 11 ]] )	
-			}
+        if ( !robust ) {
+		      if ( !heavy ) {
+            ma <- lm( target ~., data = as.data.frame( dataset[, c(da, sela, sel)] ), weights = wei, y = FALSE, model = FALSE )
+            tool[k] <- as.numeric( summary(ma)[[ 9 ]] )
+			    } else {
+			      ma <- speedglm::speedlm( target ~., data=as.data.frame( dataset[, c(da, sela, sel) ] ), weights = wei )
+            tool[k] <- as.numeric( summary( ma )[[ 11 ]] )	
+			    }
 			
-          } else {
-            ma = MASS::rlm( target ~., data = as.data.frame( dataset[, c(da, sela, sel)] ), maxit = 2000, weights = wei )
-            r2 = cor(target, fitted(ma) )^2
+        } else {
+            ma <- MASS::rlm( target ~., data = as.data.frame( dataset[, c(da, sela, sel)] ), maxit = 2000, weights = wei )
+            r2 <- cor(target, fitted(ma) )^2
             tool[k] = 1 - (1 - r2) * (n - 1) / ( n - length( coef(ma) ) - 1)
-          }               
+        }               
           if ( tool[ k ] - tool[ k - 1 ] <= tol ) {
             info <- rbind(info, c( 1e300, 0, 0 ) )
             
@@ -574,10 +554,7 @@ lm.fsreg_2 <- function(target, dataset, iniset = NULL, threshold = 0.05, wei = N
             info <- rbind( info, mat[ina, ] )
             sela <- info[, 1]
             mat <- mat[-ina , ]
-            if ( is.matrix(mat) ) {
-              mat <- mat[ order( mat[, 2] ), ]
-            }
-            
+           if ( !is.matrix(mat) )   mat <- matrix(mat, ncol = 3) 
             moda[[ k ]] <- ma
           }
           
@@ -595,18 +572,13 @@ lm.fsreg_2 <- function(target, dataset, iniset = NULL, threshold = 0.05, wei = N
   models <- NULL
 
   if ( d == 0 ) {
-    if ( robust == FALSE ) {
-	
-	  if ( heavy == FALSE ) {
-        final <- lm( target ~., data = as.data.frame( iniset ) ) 
-	  } else {
-	    final <- speedglm::speedlm( target ~., data = as.data.frame( iniset ) ) 
-	  }
+    if ( !robust ) {
+	    if ( !heavy ) {
+        final <- lm( target ~., data = as.data.frame( iniset ), y = FALSE, model = FALSE ) 
+	    } else  final <- speedglm::speedlm( target ~., data = as.data.frame( iniset ) ) 
 	  
-	} else {
-      final <- MASS::rlm( target ~., data = as.data.frame( iniset ), maxit = 2000, weights = wei ) 
-	} 
-    info = NULL
+	  } else  final <- MASS::rlm( target ~., data = as.data.frame( iniset ), maxit = 2000, weights = wei ) 
+      info <- NULL
     
   } else {
     
@@ -615,45 +587,30 @@ lm.fsreg_2 <- function(target, dataset, iniset = NULL, threshold = 0.05, wei = N
     xx <- as.data.frame( dataset[, c(da, sela) ] )
     if ( pa == 0 ) {
       colnames(xx) <- paste("V", sela, sep = "") 
-    } else {
-      colnames(xx) <- c( paste("X", da, sep = ""),  paste("V", sela, sep = "") )
-    }
+    } else  colnames(xx) <- c( paste("X", da, sep = ""),  paste("V", sela, sep = "") )
     
     if ( d == 1 ) {
       
       xx <- as.data.frame( dataset[, c(da, sela) ] )
       if ( pa == 0 ) {
         colnames(xx) <- paste("V", sela, sep = "") 
-      } else {
-        colnames(xx) <- c( paste("X", da, sep = ""),  paste("V", sela, sep = "") )
-      }
+      } else  colnames(xx) <- c( paste("X", da, sep = ""),  paste("V", sela, sep = "") )
       
-      if ( robust == FALSE ) {
-	    if ( heavy == FALSE ) {
-          models[[ 1 ]] <- lm( target ~., data = as.data.frame( xx ), weights = wei )
-		} else {
-		  models[[ 1 ]] <- speedglm::speedlm( target ~., data = as.data.frame( xx ), weights = wei )
-		}
+      if ( !robust ) {
+	      if ( !heavy ) {
+          models[[ 1 ]] <- lm( target ~., data = as.data.frame( xx ), weights = wei, y = FALSE, model = FALSE )
+		    } else  models[[ 1 ]] <- speedglm::speedlm( target ~., data = as.data.frame( xx ), weights = wei )
 		
-      } else {
-        # models[[ 1]] <- final <- robust::lmRob( target ~., data = as.data.frame( xx ), control = cont )
-        models[[ 1 ]] <- MASS::rlm( target ~., data = as.data.frame( xx ), maxit = 2000, weights = wei )
-      }
+      } else  models[[ 1 ]] <- MASS::rlm( target ~., data = as.data.frame( xx ), maxit = 2000, weights = wei )
       
     } else {
       for ( i in 1:c( pa + d ) ) {
-        if ( robust == FALSE ) {
-		  if ( heavy == FALSE ) {
-            models[[ i ]] <- lm( target ~., data = as.data.frame( xx[, 1:i] ), weights = wei )
-		  } else {
-		    models[[ i ]] <- speedglm::speedlm( target ~., data = as.data.frame( xx[, 1:i] ), weights = wei )
-		  }	
+        if ( !robust ) {
+		  if ( !heavy ) {
+          models[[ i ]] <- lm( target ~., data = as.data.frame( xx[, 1:i] ), weights = wei, y = FALSE, model = FALSE )
+		  } else  models[[ i ]] <- speedglm::speedlm( target ~., data = as.data.frame( xx[, 1:i] ), weights = wei )
 		  
-        } else { 
-          # models[[ i ]] <- lmRob( target ~., data = as.data.frame( xx[, 1:i]), control = cont )
-          models[[ i ]] <- MASS::rlm( target ~., data = data.frame( xx[, 1:i] ), maxit = 2000, weights = wei )
-          
-        }
+        } else  models[[ i ]] <- MASS::rlm( target ~., data = data.frame( xx[, 1:i] ), maxit = 2000, weights = wei )
       }
     }
     
@@ -662,11 +619,11 @@ lm.fsreg_2 <- function(target, dataset, iniset = NULL, threshold = 0.05, wei = N
     info <- info[1:d, ]
     if ( d == 1 )  info <- matrix(info, nrow = 1)
     info <- cbind( info, tool[ 1:d ] ) 
-    colnames(info) <- c( "variables", "p-value", "stat", stopping )
+    colnames(info) <- c( "variables", "log.p-value", "stat", stopping )
     rownames(info) <- info[, 1]
   }
   
   }
   
-  list(mat = t(mat), info = info, models = models, final = final, ci_test <- ci_test, runtime = runtime ) 
+  list(mat = t(mat), info = info, models = models, final = final, ci_test = ci_test, runtime = runtime ) 
 }
