@@ -32,13 +32,13 @@ testIndNB = function(target, dataset, xIndex, csIndex, wei = NULL, dataInfo=NULL
   flag = 0;
   csIndex[which(is.na(csIndex))] = 0;
   
-  if(hash == TRUE)
+  if( hash )
   {
     csIndex2 = csIndex[which(csIndex!=0)]
     csindex2 = sort(csIndex2)
     xcs = c(xIndex,csIndex2)
     key = paste(as.character(xcs) , collapse=" ");
-    if(is.null(stat_hash[[key]]) == FALSE)
+    if( !is.null(stat_hash[[key]]) )
     {
       stat = stat_hash[[key]];
       pvalue = pvalue_hash[[key]];
@@ -53,7 +53,7 @@ testIndNB = function(target, dataset, xIndex, csIndex, wei = NULL, dataInfo=NULL
   #information with respect to cs
   if(!is.na(match(xIndex,csIndex)))
   {
-    if(hash == TRUE)#update hash objects
+    if( hash )#update hash objects
     {
       stat_hash[[key]] <- 0;#.set(stat_hash , key , 0)
       pvalue_hash[[key]] <- log(1);#.set(pvalue_hash , key , 1)
@@ -77,36 +77,15 @@ testIndNB = function(target, dataset, xIndex, csIndex, wei = NULL, dataInfo=NULL
   x = dataset[ , xIndex];
   cs = dataset[ , csIndex];
   
-  #remove equal columns of the CS (takes a lot of time)
-#   if(length(csIndex) > 1)
-#   {
-#     #remove same columns
-#     #cs = unique(as.matrix(cs), MARGIN = 2);
-#     #or
-#     w = which(duplicated(cs, MARGIN = 2))
-#     if(length(w) > 0)
-#     {
-#       cs = cs[,-w]
-#     }
-#   }
-  
-  #checking the length
-  if (length(x) == 0 || length(target) == 0)
-  {
-    message(paste("error in testIndNB : empty variable x or target"))
-    results <- list(pvalue = pvalue, stat = stat, flag = flag , stat_hash=stat_hash, pvalue_hash=pvalue_hash);
-    return(results);
-  }
-  
   #if x = any of the cs then pvalue = 1 and flag = 1.
   #That means that the x variable does not add more information to our model due to an exact copy of this in the cs, so it is independent from the target
   if(length(cs)!=0)
   {
-    if(is.null(dim(cs)[2]) == TRUE) #cs is a vector
+    if( is.null(dim(cs)[2]) ) #cs is a vector
     {
       if(any(x != cs) == FALSE)  #if(!any(x == cs) == FALSE)
       {
-        if(hash == TRUE)#update hash objects
+        if( hash )#update hash objects
         {
           stat_hash[[key]] <- 0;#.set(stat_hash , key , 0)
           pvalue_hash[[key]] <- log(1);#.set(pvalue_hash , key , 1)
@@ -119,7 +98,7 @@ testIndNB = function(target, dataset, xIndex, csIndex, wei = NULL, dataInfo=NULL
       {
         if(any(x != cs[,col]) == FALSE)  #if(!any(x == cs) == FALSE)
         {
-          if(hash == TRUE)#update hash objects
+          if( hash )#update hash objects
           {
             stat_hash[[key]] <- 0;#.set(stat_hash , key , 0)
             pvalue_hash[[key]] <- log(1);#.set(pvalue_hash , key , 1)
@@ -132,9 +111,9 @@ testIndNB = function(target, dataset, xIndex, csIndex, wei = NULL, dataInfo=NULL
   }
   
   #if x or target is constant then there is no point to perform the test
-  if( var( as.numeric(x) ) == 0 || var(target) == 0)
+  if( vara( as.numeric(x) ) == 0 )
   {
-    if(hash == TRUE)#update hash objects
+    if( hash )#update hash objects
     {
       stat_hash[[key]] <- 0;#.set(stat_hash , key , 0)
       pvalue_hash[[key]] <- log(1);#.set(pvalue_hash , key , 1)
@@ -143,15 +122,12 @@ testIndNB = function(target, dataset, xIndex, csIndex, wei = NULL, dataInfo=NULL
     return(results);
   }
   
-  #remove constant columns of cs
-  cs = as.matrix(cs)
-  cs = cs[,apply(cs, 2, var, na.rm=TRUE) != 0]
-  
+
   #trycatch for dealing with errors
   res <- tryCatch(
 {
   #if the conditioning set (cs) is empty, we use a simplified formula
-  if(length(cs) == 0)
+  if( length(cs) == 0 )
   {
     #compute the relationship between x,target directly
     fit2 = MASS::glm.nb( target ~ x, weights = wei )
@@ -163,14 +139,11 @@ testIndNB = function(target, dataset, xIndex, csIndex, wei = NULL, dataInfo=NULL
     
     if(length(csIndex) == 1){
       fit1 = MASS::glm.nb( target ~ dataset[, csIndex], data = as.data.frame( dataset[,c(csIndex, xIndex)] ), weights = wei )
-    }else{
-      fit1 = MASS::glm.nb( target ~., data = as.data.frame( dataset[, csIndex] ), weights = wei )
-    }
+    }else  fit1 = MASS::glm.nb( target ~., data = as.data.frame( dataset[, csIndex] ), weights = wei )
     
     fit2 = MASS::glm.nb( target ~., data = as.data.frame( dataset[, c(csIndex, xIndex)] ), weights = wei )  
-    if ( is.null(fit2) ) { 
-      fit2 = MASS::glm.nb( target ~., data = as.data.frame( dataset[, c(xIndex, csIndex)] ), weights = wei ) 
-    }
+    if ( is.null(fit2) )  fit2 = MASS::glm.nb( target ~., data = as.data.frame( dataset[, c(xIndex, csIndex)] ), weights = wei ) 
+ 
     dev1 = fit1$deviance
     dev2 = fit2$deviance
     d1 = length( coef(fit1) )
@@ -182,14 +155,14 @@ testIndNB = function(target, dataset, xIndex, csIndex, wei = NULL, dataInfo=NULL
   flag = 1;
   
   #last error check
-  if(is.na(pvalue) || is.na(stat))
+  if( is.na(pvalue) || is.na(stat) )
   {
     pvalue = log(1);
     stat = 0;
     flag = 0;
   }else{
     #update hash objects
-    if(hash == TRUE)
+    if( hash )
     {
       stat_hash[[key]] <- stat;#.set(stat_hash , key , stat)
       pvalue_hash[[key]] <- pvalue;#.set(pvalue_hash , key , pvalue)
